# Copyright 2012 Nebula, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from vsm_dashboard.api.quantum import (Network, Subnet, Port,
                                             Router, FloatingIp)
from vsm_dashboard.api.lbaas import (Pool, Vip, Member,
                                           PoolMonitor)
from .utils import TestDataContainer

def data(TEST):
    # data returned by vsm_dashboard.api.quantum wrapper
    TEST.networks = TestDataContainer()
    TEST.subnets = TestDataContainer()
    TEST.ports = TestDataContainer()
    TEST.routers = TestDataContainer()
    TEST.q_floating_ips = TestDataContainer()
    TEST.pools = TestDataContainer()
    TEST.vips = TestDataContainer()
    TEST.members = TestDataContainer()
    TEST.monitors = TestDataContainer()

    # data return by quantumclient
    TEST.api_networks = TestDataContainer()
    TEST.api_subnets = TestDataContainer()
    TEST.api_ports = TestDataContainer()
    TEST.api_routers = TestDataContainer()
    TEST.api_q_floating_ips = TestDataContainer()
    TEST.api_pools = TestDataContainer()
    TEST.api_vips = TestDataContainer()
    TEST.api_members = TestDataContainer()
    TEST.api_monitors = TestDataContainer()

    #------------------------------------------------------------
    # 1st network
    network_dict = {'admin_state_up': True,
                    'id': '82288d84-e0a5-42ac-95be-e6af08727e42',
                    'name': 'net1',
                    'status': 'ACTIVE',
                    'subnets': ['e8abc972-eb0c-41f1-9edd-4bc6e3bcd8c9'],
                    'tenant_id': '1',
                    'router:external': False,
                    'shared': False}
    subnet_dict = {'allocation_pools': [{'end': '10.0.0.254',
                                         'start': '10.0.0.2'}],
                   'dns_nameservers': [],
                   'host_routes': [],
                   'cidr': '10.0.0.0/24',
                   'enable_dhcp': True,
                   'gateway_ip': '10.0.0.1',
                   'id': network_dict['subnets'][0],
                   'ip_version': 4,
                   'name': 'mysubnet1',
                   'network_id': network_dict['id'],
                   'tenant_id': network_dict['tenant_id']}

    TEST.api_networks.add(network_dict)
    TEST.api_subnets.add(subnet_dict)

    network = copy.deepcopy(network_dict)
    subnet = Subnet(subnet_dict)
    network['subnets'] = [subnet]
    TEST.networks.add(Network(network))
    TEST.subnets.add(subnet)

    # ports on 1st network
    port_dict = {'admin_state_up': True,
                 'device_id': 'af75c8e5-a1cc-4567-8d04-44fcd6922890',
                 'device_owner': 'network:dhcp',
                 'fixed_ips': [{'ip_address': '10.0.0.3',
                                'subnet_id': subnet_dict['id']}],
                 'id': '063cf7f3-ded1-4297-bc4c-31eae876cc91',
                 'mac_address': 'fa:16:3e:9c:d5:7e',
                 'name': '',
                 'network_id': network_dict['id'],
                 'status': 'ACTIVE',
                 'tenant_id': network_dict['tenant_id']}
    TEST.api_ports.add(port_dict)
    TEST.ports.add(Port(port_dict))

    port_dict = {'admin_state_up': True,
                 'device_id': '1',
                 'device_owner': 'compute:nova',
                 'fixed_ips': [{'ip_address': '10.0.0.4',
                                'subnet_id': subnet_dict['id']}],
                 'id': '7e6ce62c-7ea2-44f8-b6b4-769af90a8406',
                 'mac_address': 'fa:16:3e:9d:e6:2f',
                 'name': '',
                 'network_id': network_dict['id'],
                 'status': 'ACTIVE',
                 'tenant_id': network_dict['tenant_id']}
    TEST.api_ports.add(port_dict)
    TEST.ports.add(Port(port_dict))
    assoc_port = port_dict

    #------------------------------------------------------------
    # 2nd network
    network_dict = {'admin_state_up': True,
                    'id': '72c3ab6c-c80f-4341-9dc5-210fa31ac6c2',
                    'name': 'net2',
                    'status': 'ACTIVE',
                    'subnets': ['3f7c5d79-ee55-47b0-9213-8e669fb03009'],
                    'tenant_id': '2',
                    'router:external': False,
                    'shared': True}
    subnet_dict = {'allocation_pools': [{'end': '172.16.88.254',
                                         'start': '172.16.88.2'}],
                   'dns_nameservers': ['10.56.1.20', '10.56.1.21'],
                   'host_routes': [{'destination': '192.168.20.0/24',
                                    'nexthop': '172.16.88.253'},
                                   {'destination': '192.168.21.0/24',
                                    'nexthop': '172.16.88.252'}],
                   'cidr': '172.16.88.0/24',
                   'enable_dhcp': True,
                   'gateway_ip': '172.16.88.1',
                   'id': '3f7c5d79-ee55-47b0-9213-8e669fb03009',
                   'ip_version': 4,
                   'name': 'aaaa',
                   'network_id': network_dict['id'],
                   'tenant_id': network_dict['tenant_id']}

    TEST.api_networks.add(network_dict)
    TEST.api_subnets.add(subnet_dict)

    network = copy.deepcopy(network_dict)
    subnet = Subnet(subnet_dict)
    network['subnets'] = [subnet]
    TEST.networks.add(Network(network))
    TEST.subnets.add(subnet)

    port_dict = {'admin_state_up': True,
                 'device_id': '2',
                 'device_owner': 'compute:nova',
                 'fixed_ips': [{'ip_address': '172.16.88.3',
                                'subnet_id': subnet_dict['id']}],
                 'id': '1db2cc37-3553-43fa-b7e2-3fc4eb4f9905',
                 'mac_address': 'fa:16:3e:56:e6:2f',
                 'name': '',
                 'network_id': network_dict['id'],
                 'status': 'ACTIVE',
                 'tenant_id': network_dict['tenant_id']}

    TEST.api_ports.add(port_dict)
    TEST.ports.add(Port(port_dict))

    #------------------------------------------------------------
    # external network
    network_dict = {'admin_state_up': True,
                    'id': '9b466b94-213a-4cda-badf-72c102a874da',
                    'name': 'ext_net',
                    'status': 'ACTIVE',
                    'subnets': ['d6bdc71c-7566-4d32-b3ff-36441ce746e8'],
                    'tenant_id': '3',
                    'router:external': True,
                    'shared': False}
    subnet_dict = {'allocation_pools': [{'start': '172.24.4.226.',
                                         'end': '172.24.4.238'}],
                   'dns_nameservers': [],
                   'host_routes': [],
                   'cidr': '172.24.4.0/28',
                   'enable_dhcp': False,
                   'gateway_ip': '172.24.4.225',
                   'id': 'd6bdc71c-7566-4d32-b3ff-36441ce746e8',
                   'ip_version': 4,
                   'name': 'ext_subnet',
                   'network_id': network_dict['id'],
                   'tenant_id': network_dict['tenant_id']}
    ext_net = network_dict
    ext_subnet = subnet_dict

    TEST.api_networks.add(network_dict)
    TEST.api_subnets.add(subnet_dict)

    network = copy.deepcopy(network_dict)
    subnet = Subnet(subnet_dict)
    network['subnets'] = [subnet]
    TEST.networks.add(Network(network))
    TEST.subnets.add(subnet)

    #------------------------------------------------------------
    # Set up router data
    port_dict = {'admin_state_up': True,
                 'device_id': '7180cede-bcd8-4334-b19f-f7ef2f331f53',
                 'device_owner': 'network:router_gateway',
                 'fixed_ips': [{'ip_address': '10.0.0.3',
                                'subnet_id': subnet_dict['id']}],
                 'id': '44ec6726-4bdc-48c5-94d4-df8d1fbf613b',
                 'mac_address': 'fa:16:3e:9c:d5:7e',
                 'name': '',
                 'network_id': network_dict['id'],
                 'status': 'ACTIVE',
                 'tenant_id': '1'}
    TEST.api_ports.add(port_dict)
    TEST.ports.add(Port(port_dict))

    router_dict = {'id': '279989f7-54bb-41d9-ba42-0d61f12fda61',
                   'name': 'router1',
                   'external_gateway_info':
                       {'network_id': ext_net['id']},
                   'tenant_id': '1'}
    TEST.api_routers.add(router_dict)
    TEST.routers.add(Router(router_dict))
    router_dict = {'id': '10e3dc42-1ce1-4d48-87cf-7fc333055d6c',
                   'name': 'router2',
                   'external_gateway_info':
                       {'network_id': ext_net['id']},
                   'tenant_id': '1'}
    TEST.api_routers.add(router_dict)
    TEST.routers.add(Router(router_dict))

    #------------------------------------------------------------
    # floating IP
    # unassociated
    fip_dict = {'tenant_id': '1',
                'floating_ip_address': '172.16.88.227',
                'floating_network_id': ext_net['id'],
                'id': '9012cd70-cfae-4e46-b71e-6a409e9e0063',
                'fixed_ip_address': None,
                'port_id': None,
                'router_id': None}
    TEST.api_q_floating_ips.add(fip_dict)
    TEST.q_floating_ips.add(FloatingIp(fip_dict))

    # associated (with compute port on 1st network)
    fip_dict = {'tenant_id': '1',
                'floating_ip_address': '172.16.88.228',
                'floating_network_id': ext_net['id'],
                'id': 'a97af8f2-3149-4b97-abbd-e49ad19510f7',
                'fixed_ip_address': assoc_port['fixed_ips'][0]['ip_address'],
                'port_id': assoc_port['id'],
                'router_id': router_dict['id']}
    TEST.api_q_floating_ips.add(fip_dict)
    TEST.q_floating_ips.add(FloatingIp(fip_dict))

    #------------------------------------------------------------
    # LBaaS

    # 1st pool
    pool_dict = {'id': '8913dde8-4915-4b90-8d3e-b95eeedb0d49',
                 'tenant_id': '1',
                 'vip_id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                 'name': 'pool1',
                 'description': 'pool description',
                 'subnet_id': TEST.subnets.first().id,
                 'protocol': 'HTTP',
                 'lb_method': 'ROUND_ROBIN',
                 'admin_state_up': True}
    TEST.api_pools.add(pool_dict)
    TEST.pools.add(Pool(pool_dict))

    # 1st vip
    vip_dict = {'id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                'name': 'vip1',
                'address': '10.0.0.100',
                'description': 'vip description',
                'subnet_id': TEST.subnets.first().id,
                'protocol_port': '80',
                'protocol': pool_dict['protocol'],
                'pool_id': pool_dict['id'],
                'session_persistence': {'type': 'SOURCE_IP',
                                        'cookie_name': 'jssessionid'},
                'connection_limit': '10',
                'admin_state_up': True}
    TEST.api_vips.add(vip_dict)
    TEST.vips.add(Vip(vip_dict))

    # 1st member
    member_dict = {'id': '78a46e5e-eb1a-418a-88c7-0e3f5968b08',
                   'tenant_id': '1',
                   'pool_id': pool_dict['id'],
                   'address': '10.0.0.11',
                   'protocol_port': '80',
                   'weight': '10',
                   'admin_state_up': True}
    TEST.api_members.add(member_dict)
    TEST.members.add(Member(member_dict))

    # 2nd member
    member_dict = {'id': '41ac1f8d-6d9c-49a4-a1bf-41955e651f91',
                  'tenant_id': '1',
                  'pool_id': pool_dict['id'],
                  'address': '10.0.0.12',
                  'protocol_port': '80',
                  'weight': '10',
                  'admin_state_up': True}
    TEST.api_members.add(member_dict)
    TEST.members.add(Member(member_dict))

    # 2nd pool
    pool_dict = {'id': '8913dde8-4915-4b90-8d3e-b95eeedb0d50',
                 'tenant_id': '1',
                 'vip_id': 'f0881d38-c3eb-4fee-9763-12de3338041d',
                 'name': 'pool2',
                 'description': 'pool description',
                 'subnet_id': TEST.subnets.first().id,
                 'protocol': 'HTTPS',
                 'lb_method': 'ROUND_ROBIN',
                 'admin_state_up': True}
    TEST.api_pools.add(pool_dict)
    TEST.pools.add(Pool(pool_dict))

    # 1st vip
    vip_dict = {'id': 'f0881d38-c3eb-4fee-9763-12de3338041d',
                'name': 'vip2',
                'address': '10.0.0.110',
                'description': 'vip description',
                'subnet_id': TEST.subnets.first().id,
                'protocol_port': '80',
                'protocol': pool_dict['protocol'],
                'pool_id': pool_dict['id'],
                'session_persistence': {'type': 'APP_COOKIE',
                                        'cookie_name': 'jssessionid'},
                'connection_limit': '10',
                'admin_state_up': True}
    TEST.api_vips.add(vip_dict)
    TEST.vips.add(Vip(vip_dict))

    # 1st monitor
    monitor_dict = {'id': 'd4a0500f-db2b-4cc4-afcf-ec026febff96',
                    'type': 'PING',
                    'delay': '10',
                    'timeout': '10',
                    'max_retries': '10',
                    'http_method': 'GET',
                    'url_path': '/',
                    'expected_codes': '200',
                    'admin_state_up': True}
    TEST.api_monitors.add(monitor_dict)
    TEST.monitors.add(PoolMonitor(monitor_dict))

    # 2nd monitor
    monitor_dict = {'id': 'd4a0500f-db2b-4cc4-afcf-ec026febff97',
                    'type': 'PING',
                    'delay': '10',
                    'timeout': '10',
                    'max_retries': '10',
                    'http_method': 'GET',
                    'url_path': '/',
                    'expected_codes': '200',
                    'admin_state_up': True}
    TEST.api_monitors.add(monitor_dict)
    TEST.monitors.add(PoolMonitor(monitor_dict))




# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

from django.template.defaultfilters import filesizeformat
from django.utils.translation import ugettext_lazy as _

from vsm_dashboard.dashboards.admin.instances.tables import \
        AdminUpdateRow

from horizon import tables

def get_tenant_name(instance):
    return instance.tenant.name

def get_memory(instance):
    return filesizeformat(instance.flavor.ram * 1024 * 1024)

def get_vcpus(instance):
    return instance.flavor.vcpus

def get_username(instance):
    return instance.user.name

class FlockingInstancesTable(tables.DataTable):
    host = tables.Column("OS-EXT-SRV-ATTR:host", verbose_name=_("Host"))
    tenant = tables.Column(get_tenant_name, verbose_name=_("Tenant"))
    user = tables.Column(get_username, verbose_name=_("user"))
    vcpus = tables.Column(get_vcpus, verbose_name=_("VCPUs"))
    memory = tables.Column(get_memory, verbose_name=_("Memory"))
    #age = tables.Column('age', verbose_name=_("Age"), filters=(timesince,))

    class Meta:
        name = "instances"
        verbose_name = _("Instances")
        row_class = AdminUpdateRow




# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import json

from django import http
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import tabs

from vsm_dashboard.dashboards.vsm.flocking import utils
from .tables import FlockingInstancesTable
from .tabs import FlockingTabs

class IndexView(tabs.TabbedTableView):
    tab_group_class = FlockingTabs
    table_class = FlockingInstancesTable
    template_name = 'vsm/flocking/index.html'

    def get(self, request, *args, **kwargs):
        if self.request.is_ajax() and self.request.GET.get("json", False):
            try:
                instances = utils.get_instances_data(self.request)
                # Uncomment the following line to use fake test data.
                #instances = utils.get_fake_instances_data(self.request)
            except:
                instances = []
                exceptions.handle(request,
                                  _('Unable to retrieve instance list.'))
            data = json.dumps([i._apiresource._info for i in instances])
            return http.HttpResponse(data)
        else:
            return super(IndexView, self).get(request, *args, **kwargs)



# Copyright 2012 Nebula, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from datetime import timedelta

from django.conf import settings
from django.utils import datetime_safe

from keystoneclient.v2_0 import users, tenants, tokens, roles, ec2

from .utils import TestDataContainer

# Dummy service catalog with all service.
# All endpoint URLs should point to example.com.
# Try to keep them as accurate to real data as possible (ports, URIs, etc.)
SERVICE_CATALOG = [
    {"type": "compute",
     "name": "nova",
     "endpoints_links": [],
     "endpoints": [
        {"region": "RegionOne",
         "adminURL": "http://admin.nova.example.com:8774/v2",
         "internalURL": "http://int.nova.example.com:8774/v2",
         "publicURL": "http://public.nova.example.com:8774/v2"}]},
    {"type": "volume",
     "name": "nova",
     "endpoints_links": [],
     "endpoints": [
        {"region": "RegionOne",
         "adminURL": "http://admin.nova.example.com:8776/v1",
         "internalURL": "http://int.nova.example.com:8776/v1",
         "publicURL": "http://public.nova.example.com:8776/v1"}]},
    {"type": "image",
     "name": "glance",
     "endpoints_links": [],
     "endpoints": [
        {"region": "RegionOne",
         "adminURL": "http://admin.glance.example.com:9292/v1",
         "internalURL": "http://int.glance.example.com:9292/v1",
         "publicURL": "http://public.glance.example.com:9292/v1"}]},
    {"type": "identity",
     "name": "keystone",
     "endpoints_links": [],
     "endpoints": [
        {"region": "RegionOne",
         "adminURL": "http://admin.keystone.example.com:35357/v2.0",
         "internalURL": "http://int.keystone.example.com:5000/v2.0",
         "publicURL": "http://public.keystone.example.com:5000/v2.0"}]},
    {"type": "object-store",
     "name": "swift",
     "endpoints_links": [],
     "endpoints": [
        {"region": "RegionOne",
         "adminURL": "http://admin.swift.example.com:8080/",
         "internalURL": "http://int.swift.example.com:8080/",
         "publicURL": "http://public.swift.example.com:8080/"}]},
    {"type": "network",
     "name": "quantum",
     "endpoints_links": [],
     "endpoints": [
        {"region": "RegionOne",
         "adminURL": "http://admin.quantum.example.com:9696/",
         "internalURL": "http://int.quantum.example.com:9696/",
         "publicURL": "http://public.quantum.example.com:9696/"}]},
    {"type": "ec2",
     "name": "EC2 Service",
     "endpoints_links": [],
     "endpoints": [
        {"region": "RegionOne",
         "adminURL": "http://admin.nova.example.com:8773/services/Admin",
         "publicURL": "http://public.nova.example.com:8773/services/Cloud",
         "internalURL": "http://int.nova.example.com:8773/services/Cloud"}]}
]

def data(TEST):
    TEST.service_catalog = SERVICE_CATALOG
    TEST.tokens = TestDataContainer()
    TEST.users = TestDataContainer()
    TEST.tenants = TestDataContainer()
    TEST.roles = TestDataContainer()
    TEST.ec2 = TestDataContainer()

    admin_role_dict = {'id': '1',
                       'name': 'admin'}
    admin_role = roles.Role(roles.RoleManager, admin_role_dict)
    member_role_dict = {'id': "2",
                        'name': settings.OPENSTACK_KEYSTONE_DEFAULT_ROLE}
    member_role = roles.Role(roles.RoleManager, member_role_dict)
    TEST.roles.add(admin_role, member_role)
    TEST.roles.admin = admin_role
    TEST.roles.member = member_role

    user_dict = {'id': "1",
                 'name': 'test_user',
                 'email': 'test@example.com',
                 'password': 'password',
                 'token': 'test_token',
                 'enabled': True}
    user = users.User(users.UserManager(None), user_dict)
    user_dict = {'id': "2",
                 'name': 'user_two',
                 'email': 'two@example.com',
                 'password': 'password',
                 'token': 'test_token',
                 'enabled': True}
    user2 = users.User(users.UserManager(None), user_dict)
    user_dict = {'id': "3",
                 'name': 'user_three',
                 'email': 'three@example.com',
                 'password': 'password',
                 'token': 'test_token',
                 'enabled': True}
    user3 = users.User(users.UserManager(None), user_dict)
    TEST.users.add(user, user2, user3)
    TEST.user = user  # Your "current" user
    TEST.user.service_catalog = SERVICE_CATALOG

    tenant_dict = {'id': "1",
                   'name': 'test_tenant',
                   'description': "a test tenant.",
                   'enabled': True}
    tenant_dict_2 = {'id': "2",
                     'name': 'disabled_tenant',
                     'description': "a disabled test tenant.",
                     'enabled': False}
    tenant = tenants.Tenant(tenants.TenantManager, tenant_dict)
    disabled_tenant = tenants.Tenant(tenants.TenantManager, tenant_dict_2)
    TEST.tenants.add(tenant, disabled_tenant)
    TEST.tenant = tenant  # Your "current" tenant

    tomorrow = datetime_safe.datetime.now() + timedelta(days=1)
    expiration = datetime_safe.datetime.isoformat(tomorrow)

    scoped_token = tokens.Token(tokens.TokenManager,
                                dict(token={"id": "test_token_id",
                                            "expires": expiration,
                                            "tenant": tenant_dict,
                                            "tenants": [tenant_dict]},
                                     user={"id": "test_user_id",
                                           "name": "test_user",
                                           "roles": [member_role_dict]},
                                     serviceCatalog=TEST.service_catalog))
    unscoped_token = tokens.Token(tokens.TokenManager,
                                  dict(token={"id": "test_token_id",
                                              "expires": expiration},
                                       user={"id": "test_user_id",
                                             "name": "test_user",
                                             "roles": [member_role_dict]},
                                       serviceCatalog=TEST.service_catalog))
    TEST.tokens.add(scoped_token, unscoped_token)
    TEST.token = scoped_token  # your "current" token.
    TEST.tokens.scoped_token = scoped_token
    TEST.tokens.unscoped_token = unscoped_token

    access_secret = ec2.EC2(ec2.CredentialsManager, {"access": "access",
                                                     "secret": "secret"})
    TEST.ec2.add(access_secret)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import tables
from horizon.utils import functions as utils

from vsm_dashboard.api import vsm as vsmapi
from vsm_dashboard.utils import get_time_delta
from .tables import ListRBDStatusTable

LOG = logging.getLogger(__name__)


class IndexView(tables.DataTableView):
    table_class = ListRBDStatusTable
    template_name = 'vsm/rbd-status/index.html'

    def get_data(self):
        default_limit = utils.get_page_size(self.request)
        default_sort_dir = "asc"
        marker = self.request.GET.get('marker', "")

        _rbd_status = []
        try:
            _rbd_status = vsmapi.rbd_pool_status(self.request, paginate_opts={
                "limit": default_limit,
                "sort_dir": default_sort_dir,
                "marker":   marker,
            })

            if _rbd_status:
                logging.debug("resp body in view: %s" % _rbd_status)
        except:
            exceptions.handle(self.request,
                              _('Unable to retrieve sever list. '))

        rbd_status = []
        for _rbd in _rbd_status:
            rbd = {
                      "id": _rbd.id,
                      "pool": _rbd.pool,
                      "image_name": _rbd.image_name,
                      "size": _rbd.size/(1024*1024),
                      "objects": _rbd.objects,
                      "order": _rbd.order,
                      "format": _rbd.format,
                      "updated_at": get_time_delta(_rbd.updated_at),
                      }

            rbd_status.append(rbd)
        return rbd_status




# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import tables

from vsm_dashboard.api import vsm as vsmapi
from .tables import ListStorageGroupStatusTable
from django.http import HttpResponse

import json
LOG = logging.getLogger(__name__)

from vsm_dashboard.utils import get_time_delta



class ModalChartMixin(object):

   def get_context_data(self, **kwargs):
        context = super(ModalChartMixin, self).get_context_data(**kwargs)
        context['settings'] = vsmapi.get_setting_dict(self.request)
        return context

class IndexView(ModalChartMixin, tables.DataTableView):
    table_class = ListStorageGroupStatusTable
    template_name = 'vsm/storage-group-status/index.html'

    def get_data(self):
        _sgs = []
        #_sgs= vsmapi.get_sg_list(self.request,)
        try:
            _sgs = vsmapi.storage_group_status(self.request,)
            if _sgs:
                logging.debug("resp body in view: %s" % _sgs)
            settings = vsmapi.get_setting_dict(self.request)
            sg_near_full_threshold = settings['storage_group_near_full_threshold']
            sg_full_threshold = settings['storage_group_full_threshold']
        except:
            exceptions.handle(self.request,
                              _('Unable to retrieve sever list. '))

        storage_group_status = []
        for _sg in _sgs:
            sg = {"id": _sg.id,
                        "name": _sg.name,
                        "friendly_name": _sg.friendly_name,
                        "attached_pools": _sg.attached_pools,
                        "capacity_total": 0 if not _sg.capacity_total else round(_sg.capacity_total * 1.0 / 1024 / 1024, 1),
                        "capacity_used": 0 if not _sg.capacity_used else round(_sg.capacity_used * 1.0 / 1024 / 1024, 1),
                        "capacity_avail": 0 if not _sg.capacity_avail else round(_sg.capacity_avail * 1.0 / 1024 / 1024, 1),
                        "capacity_percent_used": 0 if not _sg.capacity_total else _sg.capacity_used * 10000 / _sg.capacity_total / 100.0,
                        "largest_node_capacity_used": 0 if not _sg.largest_node_capacity_used else round(_sg.largest_node_capacity_used * 1.0 / 1024 / 1024, 1),
                        "status": _sg.status,
                        "updated_at": get_time_delta(_sg.updated_at),
                        }

            if sg['capacity_percent_used'] >= int(sg_full_threshold):
                sg['color'] = "red"
                sg['warning'] = "Storage Group Full Alert"
            elif sg['capacity_percent_used'] >= int(sg_near_full_threshold):
                sg['color'] = "#ff9933"
                sg['warning'] = "Storage Group Near Full Alert"
            else:
                sg['color'] = "black"
                sg['warning'] = ""

            storage_group_status.append(sg)
        return storage_group_status

#get pie charts data
def chart_data(request):
    charts = []
    _cfg = vsmapi.get_setting_dict(None)
    _sgs = vsmapi.storage_group_status(None)

    for _sg in _sgs:
        _sg.capacity_total = 1 if not _sg.capacity_total else _sg.capacity_total

        capacity_percent_used = 0 if not _sg.capacity_total else _sg.capacity_used * 10000.0 / _sg.capacity_total / 100.0
        capacity_percent_used = round(capacity_percent_used, 2)
        capacity_percent_avail = 100 - capacity_percent_used

        #0:normal;1:near full;2:full;  
        status = 0 
        if capacity_percent_used < int(_cfg["storage_group_near_full_threshold"]):
            status = 0
        elif capacity_percent_used < int(_cfg["storage_group_full_threshold"]):
            status = 1
        else:
            status = 2

        chart = {"id": _sg.id
                ,"name":_sg.name
                ,"status":status
                ,"used":capacity_percent_used
                ,"available":capacity_percent_avail}
        charts.append(chart)

    print charts

    chartsdata = json.dumps(charts)
    return HttpResponse(chartsdata)






# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

from django.core import validators
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon.utils.validators import validate_port_range
# from horizon.utils import fields
import logging

from vsm_dashboard.api import vsm as vsm_api
from vsm_dashboard.utils.validators import validate_pool_name

LOG = logging.getLogger(__name__)

class CreateErasureCodedPool(forms.SelfHandlingForm):

    failure_url = 'horizon:vsm:poolsmanagement:index'
    name = forms.CharField(label=_("Pool name"),
                           max_length=255,
                           min_length=1,
                           error_messages={
                               'required': _('This field is required.'),
                               'invalid': _("The string may only contain"
                                            " ASCII characters and numbers.")},
                           validators=[validate_pool_name])
    tag = forms.CharField(label=_("Tag"),
                          max_length=16,
                          min_length=1,
                          error_messages={
                              'required': _('This field is required.'),})
    storage_group = forms.ChoiceField(label=_('Storage Group'))
    ec_profile = forms.ChoiceField(label=_('Erasure Coded Profile'))
    ec_failure_domain = forms.ChoiceField(label=_('Erasure Coded Failure Domain'))

    def __init__(self, request, *args, **kwargs):
        super(CreateErasureCodedPool, self).__init__(request, *args, **kwargs)
        storage_group_list = []
        ec_profile_list = []
        ec_failure_domain_list = [('osd', "OSD (default)"), ("zone", "Zone"), ('host', "Host")]
        ec_profiles = vsm_api.ec_profiles(self.request)
        for k, v in enumerate(ec_profiles):
            ec_profile_list.append((v['id'], v['name']))
        try:
            rsp, group_list= vsm_api.get_storage_group_list(self.request)
            for key in group_list:
                storage_group_list.append((key, group_list[key]))
        except:
            msg = _('Failed to get storage_group_list.')
            redirect = reverse(self.failure_url)
            exceptions.handle(request, msg, redirect=redirect)
            return False
        self.fields['storage_group'].choices = storage_group_list
        self.fields['ec_profile'].choices = ec_profile_list
        self.fields['ec_failure_domain'].choices = ec_failure_domain_list

    def handle(self, request, data):
        pass


        

class RemoveCacheTier(forms.SelfHandlingForm):

    failure_url = 'horizon:vsm:poolsmanagement:index'
    cache_tier_pool = forms.ChoiceField(label=_('Cache Tier Pool'), required=False)

    def __init__(self, request, *args, **kwargs):
        super(RemoveCacheTier, self).__init__(request, *args, **kwargs)
        cache_tier_pool_list = [('',"Select a Cache Tier Pool")]
        pools = vsm_api.pool_status(request)
        cache_tier_pool_list += [(pool.pool_id, pool.name) for pool in pools if str(pool.cache_tier_status).startswith("Cache pool for")]
        self.fields['cache_tier_pool'].choices = cache_tier_pool_list

    def handle(self, request, data):
        pass





# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

from django.conf.urls import patterns, url
from .views import IndexView,remove_ec_profiles,add_ec_profile,add_ec_profile_view
urlpatterns = patterns('',
    url(r'^$', IndexView.as_view(), name='index'),
    url(r'^add_ec_profile_view/$', add_ec_profile_view, name='add_ec_profile_view'),
    url(r'^add_ec_profile/$', add_ec_profile, name='add_ec_profile'),
    url(r'^remove_ec_profiles/$', remove_ec_profiles, name='remove_ec_profiles'),
)





# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

from django.conf.urls import patterns, url

from .views import IndexView
from .views import ServersAction
from .views import AddServersView
from .views import AddServerDetailView
from .views import RemoveServersView
from .views import AddMonitorsView
from .views import RemoveMonitorsView
from .views import StartServersView
from .views import StopServersView
from .views import ResetStatus
from .views import update_server_list
from .views import get_server_by_name
from .views import add_server

urlpatterns = patterns('',
    url(r'^$', IndexView.as_view(), name='index'),
    url(r'^addserversview/$', AddServersView.as_view(), name='addserversview'),
    url(r'^removeserversview/$', RemoveServersView.as_view(), name='removeserversview'),
    url(r'^addmonitorsview/$', AddMonitorsView.as_view(), name='addmonitorsview'),
    url(r'^removemonitorsview/$', RemoveMonitorsView.as_view(), name='removemonitorsview'),
    url(r'^startserversview/$', StartServersView.as_view(), name='startserversview'),
    url(r'^stopserversview/$', StopServersView.as_view(), name='stopserversview'),
    url(r'^servers/(?P<action>\w+)$', ServersAction, name='serversaction'),
    url(r'^reset_status/(?P<server_id>\w+)$', ResetStatus, name='reset_status'),
    url(r'^update_server_list/$', update_server_list, name='update_server_list'),

    url(r'^addserverdetailview/$', AddServerDetailView, name='addserverdetailview'),
    url(r'^get_server_by_name/$', get_server_by_name, name='get_server_by_name'),
    url(r'^add_server/$', add_server, name='add_server'),

)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.csrf import csrf_exempt
from django.shortcuts import render
from django import forms as django_froms
from horizon import exceptions
from horizon import tables
from horizon import forms
from vsm_dashboard.api import vsm as vsmapi
from .tables import ECProfilesTable
from django.http import HttpResponse,HttpResponseRedirect
from vsm_dashboard.utils import get_time_delta
import json
LOG = logging.getLogger(__name__)


class IndexView(tables.DataTableView):
    table_class = ECProfilesTable
    template_name = 'vsm/ecprofiles-management/index.html'

    def get_data(self):

        ec_profiles = vsmapi.ec_profile_get_all(self.request)
        print 'ec_profiles===%s'%ec_profiles
        return ec_profiles



def add_ec_profile_view(request):
    #print '222222222'
    template = "vsm/ecprofiles-management/add_ec_profile.html"
    context = {}
    return render(request,template,context)

def add_ec_profile(request):
    status = ""
    msg = ""
    body = json.loads(request.body)
    print body
    try:
        rsp, ret = vsmapi.ec_profile_create(request,body=body)
        ret = ret['message']
    except:
        ret = {'error_code':'-2','error_msg':'Unkown Error!'}
    resp = json.dumps(ret)
    return HttpResponse(resp)

def remove_ec_profiles(request):
    data = json.loads(request.body)
    ec_profile_id_list = data["ec_profile_id_list"]

    ec_profiles = {'ec_profiles':ec_profile_id_list}
    #print '---rbd_groups remove-%s'%rbd_groups
    #ret,message = vsmapi.rbd_remove(request, rbds)
    try:
        rsp, ret = vsmapi.ec_profiles_remove(request, ec_profiles)
        ret = ret['message']
    except:
        ret = {'error_code':'-2','error_msg':'Unkown Error!'}
    resp = json.dumps(ret)
    return HttpResponse(resp)




# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2012 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2012 Nebula, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.conf import settings
from django.test.utils import override_settings

from vsm_dashboard import api
from vsm_dashboard.test import helpers as test

class GlanceApiTests(test.APITestCase):
    @override_settings(API_RESULT_PAGE_SIZE=2)
    def test_image_list_detailed_no_pagination(self):
        # Verify that all images are returned even with a small page size
        api_images = self.images.list()
        filters = {}
        limit = getattr(settings, 'API_RESULT_LIMIT', 1000)

        glanceclient = self.stub_glanceclient()
        glanceclient.images = self.mox.CreateMockAnything()
        glanceclient.images.list(page_size=limit,
                                 limit=limit,
                                 filters=filters,).AndReturn(iter(api_images))
        self.mox.ReplayAll()

        images, has_more = api.glance.image_list_detailed(self.request)
        self.assertItemsEqual(images, api_images)
        self.assertFalse(has_more)

    def test_snapshot_list_detailed(self):
        # The total image count is under page size, should return all images.
        api_images = self.images.list()
        filters = {'property-image_type': 'snapshot'}
        limit = getattr(settings, 'API_RESULT_LIMIT', 1000)
        page_size = getattr(settings, 'API_RESULT_PAGE_SIZE', 20)

        glanceclient = self.stub_glanceclient()
        glanceclient.images = self.mox.CreateMockAnything()
        glanceclient.images.list(page_size=page_size + 1,
                                 limit=limit,
                                 filters=filters,).AndReturn(iter(api_images))
        self.mox.ReplayAll()

        images, has_more = api.glance.snapshot_list_detailed(self.request)
        self.assertItemsEqual(images, api_images)
        self.assertFalse(has_more)

    @override_settings(API_RESULT_PAGE_SIZE=2)
    def test_snapshot_list_detailed_pagination(self):
        # The total snapshot count is over page size, should return
        # page_size images.
        filters = {'property-image_type': 'snapshot'}
        page_size = settings.API_RESULT_PAGE_SIZE
        limit = getattr(settings, 'API_RESULT_LIMIT', 1000)

        api_images = self.images.list()
        images_iter = iter(api_images)

        glanceclient = self.stub_glanceclient()
        glanceclient.images = self.mox.CreateMockAnything()
        # Pass back all images, ignoring filters
        glanceclient.images.list(limit=limit,
                                 page_size=page_size + 1,
                                 filters=filters,) \
                                .AndReturn(images_iter)
        self.mox.ReplayAll()

        images, has_more = api.glance.snapshot_list_detailed(self.request)
        expected_images = api_images[:page_size]
        self.assertItemsEqual(images, expected_images)
        self.assertTrue(has_more)
        # Ensure that only the needed number of images are consumed
        # from the iterator (page_size + 1).
        self.assertEqual(len(list(images_iter)),
                         len(api_images) - len(expected_images) - 1)

    @override_settings(API_RESULT_PAGE_SIZE=2)
    def test_snapshot_list_detailed_pagination_marker(self):
        # Tests getting a second page with a marker.
        filters = {'property-image_type': 'snapshot'}
        page_size = settings.API_RESULT_PAGE_SIZE
        limit = getattr(settings, 'API_RESULT_LIMIT', 1000)
        marker = 'nonsense'

        api_images = self.images.list()[page_size:]
        images_iter = iter(api_images)

        glanceclient = self.stub_glanceclient()
        glanceclient.images = self.mox.CreateMockAnything()
        # Pass back all images, ignoring filters
        glanceclient.images.list(limit=limit,
                                 page_size=page_size + 1,
                                 filters=filters,
                                 marker=marker) \
                                .AndReturn(images_iter)
        self.mox.ReplayAll()

        images, has_more = api.glance.snapshot_list_detailed(self.request,
                                                             marker=marker)
        expected_images = api_images[:page_size]
        self.assertItemsEqual(images, expected_images)
        self.assertTrue(has_more)
        self.assertEqual(len(list(images_iter)),
                         len(api_images) - len(expected_images) - 1)




# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

from django.core import validators
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon.utils.validators import validate_port_range
import logging

from vsm_dashboard.api import vsm as vsm_api
from vsm_dashboard.utils.validators import validate_pool_name
from vsm_dashboard.utils.validators import StorageGroupValidator

LOG = logging.getLogger(__name__)

class CreateConfigurationForm(forms.SelfHandlingForm):
    failure_url = 'horizon:vsm:configuration:index'
    name = forms.CharField(label=_("Name"),
            max_length=255,
            min_length=1,
            error_messages={
            'required': _('This field is required.'),
            'invalid': _("Please enter a vaild name")},
          )
    section = forms.ChoiceField(label=_('Section'), 
          )
    default_value = forms.CharField(label=_("Default Value"),
           max_length=255,
           min_length=1,
           error_messages={
           'required': _('This field is required.'),
           'invalid': _("Please enter a vaild default value")},
          )
    current_value = forms.CharField(label=_("Current Value"),
           max_length=255,
           min_length=1,
           error_messages={
           'required': _('This field is required.'),
           'invalid': _("Please enter a vaild current value")},
          )
    def __init__(self, request, *args, **kwargs):
        super(CreateConfigurationForm, self).__init__(request, *args, **kwargs)
        section_list = [
            ('',_("Select a section")),
            ('global',_("global")),
            ('mon',_("mon")),
            ('osd',_("osd")),
            ('mds',_("mds")),
        ]
        self.fields['section'].choices = section_list

    def handle(self, request, data):
        pass


class UpdateConfigurationForm(forms.SelfHandlingForm):
    failure_url = 'horizon:vsm:configuration:index'
    name = forms.CharField(label=_("Name"),
            max_length=255,
            min_length=1,
            error_messages={
            'required': _('This field is required.'),
            'invalid': _("Please enter a vaild name")},
          )
    section = forms.ChoiceField(label=_('Section'), 
          )
    default_value = forms.CharField(label=_("Default Value"),
           max_length=255,
           min_length=1,
           error_messages={
           'required': _('This field is required.'),
           'invalid': _("Please enter a vaild default value")},
          )
    current_value = forms.CharField(label=_("Current Value"),
           max_length=255,
           min_length=1,
           error_messages={
           'required': _('This field is required.'),
           'invalid': _("Please enter a vaild current value")},
          )
    def __init__(self, request, *args, **kwargs):
        super(UpdateConfigurationForm, self).__init__(request, *args, **kwargs)
        section_list = [
            ('',_("Select a section")),
            ('global',_("global")),
            ('mon',_("mon")),
            ('osd',_("osd")),
            ('mds',_("mds")),
        ]
        self.fields['section'].choices = section_list

    def handle(self, request, data):
        pass




# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import forms

LOG = logging.getLogger(__name__)

class AddOpenstackEndpointForm(forms.SelfHandlingForm):

    failure_url = 'horizon:vsm:openstackconnect:index'

    os_tenant_name = forms.CharField(
        label = _("Tenant Name"),
        max_length = 255,
        min_length = 1,
        error_messages = {
            'required': _('This field is required.')
        }
    )
    os_username = forms.CharField(
        label = _("UserName"),
        max_length = 255,
        min_length = 1,
        error_messages = {
            'required': _('This field is required.')
        }
    )
    os_password = forms.CharField(
        label = _("Password"),
        widget=forms.PasswordInput(render_value=False),
        max_length=255,
        min_length=1,
        error_messages = {
            'required': _('This field is required.')
        }
    )
    os_auth_url = forms.CharField(
        label = _("Auth Url"),
        max_length = 255,
        min_length = 1,
        error_messages = {
            'required': _('This field is required.')
        }
    )
    os_region_name = forms.CharField(
        label = _("Region Name"),
        max_length = 255,
        min_length = 0,
        required = False
    )
    ssh_user = forms.CharField(
        label = _("SSH User Name"),
        max_length = 255,
        min_length = 1,
        error_messages = {
            'required': _('This field is required.')
        }
    )

    def handle(self, request, data):
        pass
        # TODO deliver a cluster id in data
        # data['cluster_id'] = 1
        # try:
        #     LOG.info("CEPH_LOG in ADD ip, %s" % str(data))
        #     os_tenant_name = data['os_tenant_name']
        #     os_username = data['os_username']
        #     os_password = data['os_password']
        #     os_auth_url = data['os_auth_url']
        #     ip = os_auth_url.split(":")[1][2:]
        #     appnodes = vsm_api.appnode_list(request)
        #     for appnode in appnodes:
        #         old_os_auth_url = appnode.os_auth_url
        #         old_ip = old_os_auth_url.split(":")[1][2:]
        #         if ip == old_ip:
        #             messages.error(request, "duplicate ip address")
        #             return False
        #     body = {
        #         'appnodes': {
        #             'os_tenant_name': os_tenant_name,
        #             'os_username': os_username,
        #             'os_password': os_password,
        #             'os_auth_url': os_auth_url
        #         }
        #     }
        #     LOG.info("CEPH_LOG in handle body %s" % str(body))
        #     ret = vsm_api.add_appnodes(request, body['appnodes'])
        #
        #     messages.success(request,
        #                      _('Successfully add openstack: %s')
        #                      % data['os_auth_url'])
        #     return ret
        # except:
        #     redirect = reverse("horizon:vsm:openstackconnect:index")
        #     exceptions.handle(request,
        #                       _('Unable to create appnode.'),
        #                       redirect=redirect)

class UpdateOpenstackEndpointForm(forms.SelfHandlingForm):
    id = forms.CharField(label=_("ID"), widget=forms.HiddenInput)

    os_tenant_name = forms.CharField(
        label = _("Tenant Name"),
        max_length = 255,
        min_length = 1,
        error_messages = {
            'required': _('This field is required.')
        }
    )
    os_username = forms.CharField(
        label = _("UserName"),
        max_length = 255,
        min_length = 1,
        error_messages = {
            'required': _('This field is required.')
        }
    )
    os_password = forms.CharField(
        label = _("Password"),
        widget=forms.PasswordInput(render_value=False),
        max_length=255,
        min_length=1,
        error_messages = {
            'required': _('This field is required.')
        }
    )
    os_auth_url = forms.CharField(
        label = _("Auth Url"),
        max_length = 255,
        min_length = 1,
        error_messages = {
            'required': _('This field is required.')
        }
    )
    os_region_name = forms.CharField(
        label = _("Region Name"),
        max_length = 255,
        min_length = 0,
        required = False
    )
    ssh_user = forms.CharField(
        label = _("SSH User Name"),
        max_length = 255,
        min_length = 1,
        error_messages = {
            'required': _('This field is required.')
        }
    )

    def handle(self, request, data):
        pass
        # failed, succeeded = [], []
        # id = data.pop('id')
        # # ip = data.pop('ip')
        # os_tenant_name = data.pop('os_tenant_name')
        # os_username = data.pop('os_username')
        # os_password = data.pop('os_password')
        # os_auth_url = data.pop('os_auth_url')
        # vsm_api.update_appnode(request, id,
        #                        os_tenant_name=os_tenant_name,
        #                        os_username=os_username,
        #                        os_password=os_password,
        #                        os_auth_url=os_auth_url,
        #                        ssh_status="",
        #                        log_info="")
        #
        # messages.success(request, _('OpenStack auth has been updated successfully.'))
        # return True
        #
        # if failed:
        #     failed = map(force_unicode, failed)
        #     messages.error(request,
        #                    _('Unable to update %(attributes)s for the user.')
        #                      % {"attributes": ", ".join(failed)})
        # return True



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
URL patterns for the VSM Dashboard.
"""

from django.conf.urls import patterns, url, include
from django.conf.urls.static import static
from django.conf import settings
from django.contrib.staticfiles.urls import staticfiles_urlpatterns

import horizon
from django.views.generic import RedirectView

urlpatterns = patterns('',
    #url(r'^$', 'vsm_dashboard.views.splash', name='splash'),
    url(r'^$', RedirectView.as_view(url='/dashboard/vsm/')),
    url(r'^auth/', include('openstack_auth.urls')),
    url(r'^license_accept/', 'vsm_dashboard.views.license_accept',
        name='license_accept'),
    url(r'^license_cancel/', 'vsm_dashboard.views.license_cancel',
        name='license_cancel'),
    url(r'^home/', RedirectView.as_view(url='/dashboard/vsm/')),
    url(r'', include(horizon.urls))
)

# Development static app and project media serving using the staticfiles app.
urlpatterns += staticfiles_urlpatterns()

# Convenience function for serving user-uploaded media during
# development. Only active if DEBUG==True and the URL prefix is a local
# path. Production media should NOT be served by Django.
#urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)

if settings.DEBUG:
    urlpatterns += patterns('',
        url(r'^500/$', 'django.views.defaults.server_error')
    )




# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

from django.conf.urls import patterns, url
from .views import IndexView
from .views import ImportClusterView
from .views import import_cluster
from .views import auto_detect
from .views import validate_conf
#from .views import check_cluster_tobe_import

urlpatterns = patterns('',
    url(r'^$', IndexView.as_view(), name='index'),
    url(r'^importcluster/$', ImportClusterView.as_view(), name='importcluster'),
    url(r'^import_cluster/$', import_cluster, name='import_cluster'),
    url(r'^auto_detect/$', auto_detect, name='auto_detect'),
    url(r'^validate_conf/$', validate_conf, name='validate_conf'),
    #url(r'^check_cluster_tobe_import/$', check_cluster_tobe_import, name='check_cluster_tobe_import'),
)


# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.template.defaultfilters import filesizeformat
from django.utils.translation import ugettext_lazy as _
from django.utils.datastructures import SortedDict
from django import forms

from django.utils.safestring import mark_safe

from horizon import tables
from horizon.utils import html
from horizon import exceptions
from vsm_dashboard.api import vsm as vsmapi
from .utils import checkbox_transform

STRING_SEPARATOR = "__"

class UpdateRow(tables.Row):
    ajax = True

    def get_data(self, request, osd_id):
        osd = vsmapi.osd_get(request, osd_id)
        return osd

STATUS_DISPLAY_CHOICES = (
    ("resize", "Resize/Migrate"),
    ("verify_resize", "Confirm or Revert Resize/Migrate"),
    ("revert_resize", "Revert Resize/Migrate"),
)

class ListOSDStatusTable(tables.DataTable):
    STATUS_CHOICES = (
        ("Present", True),
        ("Removed", False),
    )

    #server_id = tables.Column("id", verbose_name=_("ID"))
    ordinal = tables.Column("id", verbose_name=_("ordinal"), hidden=True)
    osd_name = tables.Column("osd_name", verbose_name=_("OSD Name"))
    vsm_status = tables.Column("vsm_status",
                               status=True,
                               status_choices=STATUS_CHOICES,
                               display_choices=STATUS_DISPLAY_CHOICES,
                               verbose_name=_("VSM Status"),)
    osd_state = tables.Column("osd_state", verbose_name=_("OSD Status"))
    crush_weight = tables.Column("crush_weight", verbose_name=_("Crush Weight"))
    capacity_total = tables.Column("capacity_total", verbose_name=_("Capacity Total (MB)"))
    capacity_used = tables.Column("capacity_used", verbose_name=_("Capacity Used (MB)"))
    capacity_avail = tables.Column("capacity_avail", verbose_name=_("Capacity Available (MB)"))
    capacity_percent_used = tables.Column("capacity_percent_used", verbose_name=_("Percent Used Capacity"))
    server = tables.Column("server", verbose_name=_("Server"))
    storage_group = tables.Column("storage_group", verbose_name=_("Storage Group"))
    zone = tables.Column("zone", verbose_name=_("Zone"))
    updated_at = tables.Column("updated_at", verbose_name=_("Updated at"), classes=("span2",))
    deviceInfo = tables.Column("deviceInfo", verbose_name=_("deviceInfo"), classes=("deviceInfo",))
    page_index = tables.Column("page_index", verbose_name=_("page_index"), classes=("page_index",),hidden=True)
    page_count = tables.Column("page_count", verbose_name=_("page_count"), classes=("page_count",),hidden=True)
    pager_index = tables.Column("pager_index", verbose_name=_("pager_index"),classes=("pager_index",),hidden=True)
    pager_count = tables.Column("pager_count", verbose_name=_("pager_count"),classes=("pager_count",),hidden=True)

    class Meta:
        name = "server_list"
        verbose_name = _("OSD List")
        #status_columns = ['status']
        #row_class = UpdateRow
        multi_select = False

    def get_object_id(self, datum):
        if hasattr(datum, "id"):
            return datum.id
        else:
            return datum["id"]


def empty_value_maker(type, name, value, attrs=None):
    def _empty_value_caller(datum):
        if type == "text":
            widget = forms.TextInput()
        elif type == "choice":
            widget = forms.ChoiceField().widget
        elif type == "checkbox":
            widget = forms.CheckboxInput()
        data = dict(name=name, value=value)
        if name in datum.keys():
            data.update(datum[name])
        if attrs:
            data.update(dict(attrs=attrs))
        data = widget.render(**data)
        return data
    return _empty_value_caller





# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import logging

from horizon import forms
from horizon.utils import validators

from django.forms import ValidationError
from django.conf import settings
try:
    from django.utils.translation import force_unicode
except:
    from django.utils.translation import force_text as force_unicode
from django.utils.translation import  ugettext_lazy as _

from vsm_dashboard import api
from vsm_dashboard.utils.validators import validate_user_name
from vsm_dashboard.utils.validators import password_validate_regrex

LOG = logging.getLogger(__name__)


class BaseUserForm(forms.SelfHandlingForm):
    def __init__(self, request, *args, **kwargs):
        super(BaseUserForm, self).__init__(request, *args, **kwargs)
        # Populate tenant choices

    def clean(self):
        '''Check to make sure password fields match.'''
        data = super(forms.Form, self).clean()
        if 'password' in data:

            if data['password'] != data.get('confirm_password', None):
                raise ValidationError(_('Passwords do not match.'))
        return data

class CreateUserForm(BaseUserForm):

    failure_url = 'horizon:vsm:usermgmt:index'
    name = forms.CharField(label=_("User name"),
           max_length=255,
           min_length=1,
           error_messages={
           'required': _('This field is required.'),
           'invalid': _("Please enter a vaild User Name")},
           validators= [validate_user_name,]
           )
    password = forms.RegexField(label=_("Password"),
            widget=forms.PasswordInput(render_value=False),
            regex=password_validate_regrex,
            required=False,
            max_length=255,
            min_length=8,
            error_messages={'invalid':
                    validators.password_validator_msg()})
    confirm_password = forms.RegexField(label=_("Confirm Password"),
            widget=forms.PasswordInput(render_value=False),
            regex=validators.password_validator(),
            required=False,
            error_messages={'invalid':
                    validators.password_validator_msg()})
#    email = forms.CharField(label=_("Email"),
#            max_length=255,
#            min_length=1,
#            required=False,
#            error_messages={
#                'required': _('This field is required.'),
#                'invalid': _('Please enter an email address.'),
#            },)

    def handle(self, request, data):
        pass


class UpdateUserForm(BaseUserForm):
    id = forms.CharField(label=_("ID"), widget=forms.HiddenInput)
    #name = forms.CharField(label=_("User Name"))
    #email = forms.EmailField(label=_("Email"))
    password = forms.RegexField(label=_("Password"),
            widget=forms.PasswordInput(render_value=False),
            regex=password_validate_regrex,
            required=False,
            error_messages={'invalid':
                    validators.password_validator_msg()})
    confirm_password = forms.CharField(
            label=_("Confirm Password"),
            widget=forms.PasswordInput(render_value=False),
            required=False)

    def __init__(self, request, *args, **kwargs):
        super(UpdateUserForm, self).__init__(request, *args, **kwargs)

        if api.keystone.keystone_can_edit_user() is False:
            for field in ('name', 'email', 'password', 'confirm_password'):
                self.fields.pop(field)

    def handle(self, request, data):
        pass


class UserSettingsForm(forms.SelfHandlingForm):
    pagesize = forms.IntegerField(label=_("Items Per Page"),
                                  min_value=1,
                                  max_value=getattr(settings,
                                                    'API_RESULT_LIMIT',
                                                    1000))

    def __init__(self, *args, **kwargs):
        super(UserSettingsForm, self).__init__(*args, **kwargs)


    def handle(self, request, data):
        pass


# vim: tabstop=4 shiftwidth=4 softtabstop=4

#
# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import

import logging
from vsmclient.v1 import client as vsm_client
from vsmclient.v1.pool_usages import PoolUsageManager
from vsmclient.v1.appnodes import AppNodeManager
from vsm_dashboard.api.base import APIResourceWrapper
from django.conf import settings

LOG = logging.getLogger(__name__)

class ExtensionManager:
    def __init__(self, name, manager_class):
        self.name = name
        self.manager_class = manager_class

def vsmclient(request):
    key_vsm_pass = getattr(settings,'KEYSTONE_VSM_SERVICE_PASSWORD')
    key_url = getattr(settings, 'OPENSTACK_KEYSTONE_URL')
    c = vsm_client.Client('vsm',
                          key_vsm_pass,
                          'service',
                          key_url,
                          extensions=[ExtensionManager('PoolUsageManager',
                                                PoolUsageManager),
                                      ExtensionManager('AppNodeManager',
                                                AppNodeManager)])
    return c

class Pool(APIResourceWrapper):
    """Simple wrapper around vsmclient.vsms
    """
    _attrs = ['id', 'name', 'status', 'recipe_id', 'deleted']

    def __init__(self, apiresource, request):
        super(Pool, self).__init__(apiresource)
        self.request = request

def create_storage_pool(request, body):
    return vsmclient(request).vsms.create_storage_pool(body)

def add_cache_tier(request, body):
    return vsmclient(request).storage_pools.add_cache_tier(body)

def remove_cache_tier(request, body):
    return vsmclient(request).storage_pools.remove_cache_tier(body)

def get_storage_group_list(request):
    return vsmclient(request).vsms.get_storage_group_list()

def osd_locations_choices_by_type(request):
    return vsmclient(request).zones.osd_locations_choices()

def get_pool_size_list(request):
    return vsmclient(request).vsms.get_pool_size_list()

def pools_list(request, search_opts=None, all_pools=False):
    if search_opts is None:
        search_opts = {}
    if all_pools:
        search_opts['all_pools'] = True
    return vsmclient(request).vsms.list_storage_pool(request, search_opts)

def pool_list(request):
    search_opts = {}
    r, pool_list = vsmclient(request).vsms.list_storage_pool(request, search_opts)
    return pool_list['pool']

def pool_usages(request):
    return vsmclient(request).PoolUsageManager.list()

def present_pool(request, pools):
    return vsmclient(request).PoolUsageManager.create(pools=pools)

def add_appnodes(request, auth_openstack):
    return vsmclient(request).AppNodeManager.create(auth_openstack)

def del_appnode(request, appnode_id):
    return vsmclient(request).AppNodeManager.delete(appnode_id)

def update_appnode(request, appnode, **kwargs):
    return vsmclient(request).AppNodeManager.update(appnode, **kwargs)

def appnode_list(request,):
    return vsmclient(request).AppNodeManager.list()

resp = None
#server api
def add_servers(request, servers=[]):
    return vsmclient(request).servers.add(servers)

def remove_servers(request, servers=[]):
    return vsmclient(request).servers.remove(servers)

def reset_status(request, servers):
    return vsmclient(request).servers.reset_status(servers)

def get_server_list(request):
    return vsmclient(request).servers.list()

def get_server(request, id):
    return vsmclient(request).servers.get(id)

def start_server(request, servers=None):
    """Start servers.
       servers = [{'id': 1}, {'id': 2}]
    """
    return vsmclient(request).servers.start(servers)


def stop_server(request, servers=None):
    """Stop servers.
       servers = [{'id': 1}, {'id': 2}]
    """
    LOG.debug("DEBUG in stop server of dashboard api")
    return vsmclient(request).servers.stop(servers)

def ceph_upgrade(request, body=None):
    """ceph_upgrade.
       body = {         'key_url':"https://...",
                        'proxy':"https://...",
                        'pkg_url':"https://..."}}
    """
    return vsmclient(request).servers.ceph_upgrade(body)

#zone api
def get_zone_list(request):
    return vsmclient(request).zones.list()

#zone api
def get_zone_not_in_crush_list(request):
    return vsmclient(request).zones.get_zone_not_in_crush_list()


# TODO return the exact response
def create_zone(request, opts=None):
    res = vsmclient(request).zones.create(opts)
    return resp, {'status': "succes", "message": "create zone success"}

#cluster api
def get_cluster_list(request, opts=None):
    return vsmclient(request).vsms.get_cluster_list()

def create_cluster(request, servers=[]):
    return vsmclient(request).clusters.create(servers=servers)

def get_service_list(request):
    return vsmclient(request).clusters.get_service_list()

def integrate_cluster(request, servers=[]):
    return vsmclient(request).clusters.integrate(servers=servers)
#osd api
def osd_list(request):
    return vsmclient(request).osds.list()

def osd_get(request, osd_id):
    return vsmclient(request).osds.get(osd_id)

def osd_restart(request, osd_id):
    return vsmclient(request).osds.restart(osd_id)

def osd_delete(request, osd_id):
    return vsmclient(request).osds.delete(osd_id)

def osd_remove(request, osd_id):
    return vsmclient(request).osds.remove(osd_id)

def osd_restore(request, osd_id):
    return vsmclient(request).osds.restore(osd_id)

def osd_status(request, paginate_opts=None):
    return vsmclient(request).osds.list(detailed=True, paginate_opts=paginate_opts)

def osd_status_sort_and_filter(request, paginate_opts=None):
    return vsmclient(request).osds.list(detailed='detail_filter_and_sort', paginate_opts=paginate_opts)

def osd_summary(request):
    return vsmclient(request).osds.summary()

def mds_status(request):
    return vsmclient(request).mdses.list(detailed=True)

def monitor_summary(request):
    return vsmclient(request).monitors.summary()

def monitor_status(request):
    return vsmclient(request).monitors.list(detailed=True)

def storage_group_summary(request):
    return vsmclient(request).storage_groups.summary()

def storage_group_status(request):
    return vsmclient(request).storage_groups.list(detailed=True)

def storage_group_create(request, body):
    return vsmclient(request).storage_groups.create(body)

def storage_group_create_with_takes(request, body):
    return vsmclient(request).storage_groups.create_with_takes(body)

def storage_group_update_with_takes(request, body):
    return vsmclient(request).storage_groups.update_with_takes(body)

def storage_group_update(request, body):
    return vsmclient(request).storage_groups.update(body)

def get_default_pg_num_by_storage_group(request, body):
    return vsmclient(request).storage_groups.get_default_pg_num(search_opts=body)

def placement_group_summary(request):
    return vsmclient(request).placement_groups.summary()

def placement_group_status(request, paginate_opts=None):
    return vsmclient(request).placement_groups.list(detailed=True,
                                                    paginate_opts=paginate_opts)

def rbd_pool_summary(request):
    return vsmclient(request).rbd_pools.summary()

def rbd_pool_status(request, paginate_opts=None):
    return vsmclient(request).rbd_pools.list(detailed=True, paginate_opts=paginate_opts)

def mds_summary(request):
    return vsmclient(request).mdses.summary()

def cluster_summary(request):
    return vsmclient(request).clusters.summary()

def vsm_summary(request):
    return vsmclient(request).vsms.summary()

def pool_status(request):
    return vsmclient(request).storage_pools.list(detailed=True)

def ec_profiles(request):
    return vsmclient(request).storage_pools.ec_profiles()

def ec_profiles_remove(request,body):
    return vsmclient(request).ec_profiles.ec_profiles_remove(body=body)

def ec_profile_create(request,body):
    return vsmclient(request).ec_profiles.ec_profile_create(body=body)

def ec_profile_get_all(request):
    return vsmclient(request).ec_profiles.list()

#device api
def device_list(request):
    return vsmclient(request).devices.list()

def device_get_smartinfo(request,search_opts=None):
    ret = vsmclient(request).devices.get_smart_info(search_opts=search_opts)
    return ret

#license api
def license_create(request, value=True):
    return vsmclient(request).licenses.license_create(value)

def license_get(request):
    return vsmclient(request).licenses.license_get()

def license_update(request, value):
    return vsmclient(request).licenses.license_update(value)

def get_setting_dict(request,):
    # TODO
    setting_list = vsmclient(request).vsm_settings.list()
    setting_dict = {}
    for setting in setting_list:
        setting_dict.setdefault(setting.name, setting.value)
    return setting_dict

def get_settings(request,):
    return vsmclient(request).vsm_settings.list()

def get_setting_by_name(request, name):
    return vsmclient(request).vsm_settings.get(name)

def update_setting(request, name, value):
    return vsmclient(request).vsm_settings.create({'name': name, 'value':value})

def get_metrics(request,search_opts):
    return vsmclient(request).performance_metrics.get_metrics(search_opts=search_opts)

def get_metrics_all_types(request,search_opts):
    return vsmclient(request).performance_metrics.get_metrics_all_types(search_opts=search_opts)

def add_osd_from_node_in_cluster(request,osd_states_id):
    return  vsmclient(request).osds.add_osd_from_node_in_cluster(osd_states_id)

def get_ceph_health_list(request):
    return  vsmclient(request).clusters.get_ceph_health_list()

def add_new_disks_to_cluster(request, body):
    return vsmclient(request).osds.add_new_disks_to_cluster(body)

def add_batch_new_disks_to_cluster(request, body):
    return vsmclient(request).osds.add_batch_new_disks_to_cluster(body)


def get_available_disks(request,  search_opts):
    return vsmclient(request).devices.get_available_disks( search_opts=search_opts)

def check_pre_existing_cluster(request, body=None):
    """check_pre_existing_cluster.
        body : {u'cluster_conf': u'/etc/ceph/ceph.conf',
        u'monitor_host_name': u'centos-storage1',
        u'monitor_host_id': u'1',
         u'monitor_keyring': u'/etc/keying'}
    """
    return vsmclient(request).clusters.check_pre_existing_cluster(body)


def import_cluster(request, body=None):
    """check_pre_existing_cluster.
        body : {u'cluster_conf': u'/etc/ceph/ceph.conf',
        u'monitor_host_name': u'centos-storage1',
        u'monitor_host_id': u'1',
         u'monitor_keyring': u'/etc/keying'}
    """
    return vsmclient(request).clusters.import_cluster(body)

def detect_crushmap(request, body=None):
    """check_pre_existing_cluster.
        body : {u'cluster_conf': u'/etc/ceph/ceph.conf',
        u'monitor_host_name': u'centos-storage1',
        u'monitor_host_id': u'1',
         u'monitor_keyring': u'/etc/keying'}
    """
    return vsmclient(request).clusters.detect_crushmap(body)

def get_crushmap_tree_data(request, body=None):
    """
        body : {'cluster_id':1}
    """
    return vsmclient(request).clusters.get_crushmap_tree_data(body)

def add_zone_to_crushmap_and_db(request, body):
    '''

    :param request:
    :param body:
    {'zone_name':'kkk',
    'zone_parent_type':'host',
    'zone_parent_name':'parent_zone'}
    :return:
    '''
    return vsmclient(request).zones.add_zone_to_crushmap_and_db(body)

def detect_cephconf(request, body=None):
    """check_pre_existing_cluster.
        body : {u'cluster_conf': u'/etc/ceph/ceph.conf',
        u'monitor_host_name': u'centos-storage1',
        u'monitor_host_id': u'1',
         u'monitor_keyring': u'/etc/keying'}
    """
    return vsmclient(request).clusters.detect_cephconf(body)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from vsmclient import exceptions as vsmclient
from keystoneclient import exceptions as keystoneclient

UNAUTHORIZED = (keystoneclient.Unauthorized,
                keystoneclient.Forbidden,
                vsmclient.Unauthorized,
                vsmclient.Forbidden)

NOT_FOUND = (keystoneclient.NotFound,
             vsmclient.NotFound)

# NOTE(gabriel): This is very broad, and may need to be dialed in.
RECOVERABLE = (keystoneclient.ClientException,
               # AuthorizationFailure is raised when Keystone is "unavailable".
               keystoneclient.AuthorizationFailure,
               vsmclient.ClientException)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2012 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2012 Nebula, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import

from mox import IsA

from vsm_dashboard import api
from horizon import exceptions
from vsm_dashboard.test import helpers as test

class SwiftApiTests(test.APITestCase):
    def test_swift_get_containers(self):
        containers = self.containers.list()
        cont_data = [c._apidict for c in containers]
        swift_api = self.stub_swiftclient()
        swift_api.get_account(limit=1001,
                              marker=None,
                              full_listing=True).AndReturn([{}, cont_data])
        self.mox.ReplayAll()

        (conts, more) = api.swift.swift_get_containers(self.request)
        self.assertEqual(len(conts), len(containers))
        self.assertFalse(more)

    def test_swift_create_duplicate_container(self):
        container = self.containers.first()
        swift_api = self.stub_swiftclient(expected_calls=2)
        # Check for existence, then create
        exc = self.exceptions.swift
        swift_api.head_container(container.name).AndRaise(exc)
        swift_api.put_container(container.name).AndReturn(container)
        self.mox.ReplayAll()
        # Verification handled by mox, no assertions needed.
        api.swift.swift_create_container(self.request, container.name)

    def test_swift_create_container(self):
        container = self.containers.first()
        swift_api = self.stub_swiftclient()
        swift_api.head_container(container.name).AndReturn(container)
        self.mox.ReplayAll()
        # Verification handled by mox, no assertions needed.
        with self.assertRaises(exceptions.AlreadyExists):
            api.swift.swift_create_container(self.request, container.name)

    def test_swift_get_objects(self):
        container = self.containers.first()
        objects = self.objects.list()

        swift_api = self.stub_swiftclient()
        swift_api.get_container(container.name,
                                limit=1001,
                                marker=None,
                                prefix=None,
                                delimiter='/',
                                full_listing=True).AndReturn([{}, objects])
        self.mox.ReplayAll()

        (objs, more) = api.swift.swift_get_objects(self.request,
                                                   container.name)
        self.assertEqual(len(objs), len(objects))
        self.assertFalse(more)

    def test_swift_upload_object(self):
        container = self.containers.first()
        obj = self.objects.first()
        fake_name = 'fake_object.jpg'

        class FakeFile(object):
            def __init__(self):
                self.name = fake_name
                self.data = obj.data
                self.size = len(obj.data)

        headers = {'X-Object-Meta-Orig-Filename': fake_name}

        swift_api = self.stub_swiftclient()
        swift_api.put_object(container.name,
                             obj.name,
                             IsA(FakeFile),
                             headers=headers)
        self.mox.ReplayAll()

        api.swift.swift_upload_object(self.request,
                                      container.name,
                                      obj.name,
                                      FakeFile())

    def test_swift_object_exists(self):
        container = self.containers.first()
        obj = self.objects.first()

        swift_api = self.stub_swiftclient(expected_calls=2)
        swift_api.head_object(container.name, obj.name).AndReturn(container)

        exc = self.exceptions.swift
        swift_api.head_object(container.name, obj.name).AndRaise(exc)
        self.mox.ReplayAll()

        args = self.request, container.name, obj.name
        self.assertTrue(api.swift.swift_object_exists(*args))
        # Again, for a "non-existent" object
        self.assertFalse(api.swift.swift_object_exists(*args))



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2012 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2012 Nebula, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import

from horizon import exceptions

from vsm_dashboard.test import helpers as test
from vsm_dashboard.api import base as api_base

class APIResource(api_base.APIResourceWrapper):
    """ Simple APIResource for testing """
    _attrs = ['foo', 'bar', 'baz']

    @staticmethod
    def get_instance(innerObject=None):
        if innerObject is None:

            class InnerAPIResource(object):
                pass

            innerObject = InnerAPIResource()
            innerObject.foo = 'foo'
            innerObject.bar = 'bar'
        return APIResource(innerObject)

class APIDict(api_base.APIDictWrapper):
    """ Simple APIDict for testing """
    _attrs = ['foo', 'bar', 'baz']

    @staticmethod
    def get_instance(innerDict=None):
        if innerDict is None:
            innerDict = {'foo': 'foo',
                         'bar': 'bar'}
        return APIDict(innerDict)

# Wrapper classes that only define _attrs don't need extra testing.
class APIResourceWrapperTests(test.TestCase):
    def test_get_attribute(self):
        resource = APIResource.get_instance()
        self.assertEqual(resource.foo, 'foo')

    def test_get_invalid_attribute(self):
        resource = APIResource.get_instance()
        self.assertNotIn('missing', resource._attrs,
                msg="Test assumption broken.  Find new missing attribute")
        with self.assertRaises(AttributeError):
            resource.missing

    def test_get_inner_missing_attribute(self):
        resource = APIResource.get_instance()
        with self.assertRaises(AttributeError):
            resource.baz

class APIDictWrapperTests(test.TestCase):
    # APIDict allows for both attribute access and dictionary style [element]
    # style access.  Test both
    def test_get_item(self):
        resource = APIDict.get_instance()
        self.assertEqual(resource.foo, 'foo')
        self.assertEqual(resource['foo'], 'foo')

    def test_get_invalid_item(self):
        resource = APIDict.get_instance()
        self.assertNotIn('missing', resource._attrs,
                msg="Test assumption broken.  Find new missing attribute")
        with self.assertRaises(AttributeError):
            resource.missing
        with self.assertRaises(KeyError):
            resource['missing']

    def test_get_inner_missing_attribute(self):
        resource = APIDict.get_instance()
        with self.assertRaises(AttributeError):
            resource.baz
        with self.assertRaises(KeyError):
            resource['baz']

    def test_get_with_default(self):
        resource = APIDict.get_instance()

        self.assertEqual(resource.get('foo'), 'foo')

        self.assertIsNone(resource.get('baz'))

        self.assertEqual('retValue', resource.get('baz', 'retValue'))

class ApiHelperTests(test.TestCase):
    """ Tests for functions that don't use one of the api objects """

    def test_url_for(self):
        url = api_base.url_for(self.request, 'image')
        self.assertEqual(url, 'http://public.glance.example.com:9292/v1')

        url = api_base.url_for(self.request, 'image', admin=False)
        self.assertEqual(url, 'http://public.glance.example.com:9292/v1')

        url = api_base.url_for(self.request, 'image', admin=True)
        self.assertEqual(url, 'http://admin.glance.example.com:9292/v1')

        url = api_base.url_for(self.request, 'compute')
        self.assertEqual(url, 'http://public.nova.example.com:8774/v2')

        url = api_base.url_for(self.request, 'compute', admin=False)
        self.assertEqual(url, 'http://public.nova.example.com:8774/v2')

        url = api_base.url_for(self.request, 'compute', admin=True)
        self.assertEqual(url, 'http://admin.nova.example.com:8774/v2')

        url = api_base.url_for(self.request, 'volume')
        self.assertEqual(url, 'http://public.nova.example.com:8776/v1')

        url = api_base.url_for(self.request, 'volume',
                               endpoint_type="internalURL")
        self.assertEqual(url, 'http://int.nova.example.com:8776/v1')

        url = api_base.url_for(self.request, 'volume', admin=False)
        self.assertEqual(url, 'http://public.nova.example.com:8776/v1')

        url = api_base.url_for(self.request, 'volume', admin=True)
        self.assertEqual(url, 'http://admin.nova.example.com:8776/v1')

        self.assertNotIn('notAnApi', self.request.user.service_catalog,
                         'Select a new nonexistent service catalog key')
        with self.assertRaises(exceptions.ServiceCatalogException):
            url = api_base.url_for(self.request, 'notAnApi')



# Copyright 2012 Nebula, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from glanceclient.v1.images import Image, ImageManager

from .utils import TestDataContainer

def data(TEST):
    TEST.images = TestDataContainer()
    TEST.snapshots = TestDataContainer()

    # Snapshots
    snapshot_dict = {'name': u'snapshot',
                     'container_format': u'ami',
                     'id': 3,
                     'status': "active",
                     'owner': TEST.tenant.id,
                     'properties': {'image_type': u'snapshot'},
                     'is_public': False}
    snapshot_dict_no_owner = {'name': u'snapshot 2',
                              'container_format': u'ami',
                              'id': 4,
                              'status': "active",
                              'owner': None,
                              'properties': {'image_type': u'snapshot'},
                              'is_public': False}
    snapshot_dict_queued = {'name': u'snapshot 2',
                            'container_format': u'ami',
                            'id': 5,
                            'status': "queued",
                            'owner': TEST.tenant.id,
                            'properties': {'image_type': u'snapshot'},
                            'is_public': False}
    snapshot = Image(ImageManager(None), snapshot_dict)
    TEST.snapshots.add(snapshot)
    snapshot = Image(ImageManager(None), snapshot_dict_no_owner)
    TEST.snapshots.add(snapshot)
    snapshot = Image(ImageManager(None), snapshot_dict_queued)
    TEST.snapshots.add(snapshot)

    # Images
    image_dict = {'id': '007e7d55-fe1e-4c5c-bf08-44b4a4964822',
                  'name': 'public_image',
                  'status': "active",
                  'owner': TEST.tenant.id,
                  'container_format': 'novaImage',
                  'properties': {'image_type': u'image'},
                  'is_public': True}
    public_image = Image(ImageManager(None), image_dict)

    image_dict = {'id': 'a001c047-22f8-47d0-80a1-8ec94a9524fe',
                  'name': 'private_image',
                  'status': "active",
                  'owner': TEST.tenant.id,
                  'container_format': 'aki',
                  'is_public': False}
    private_image = Image(ImageManager(None), image_dict)

    image_dict = {'id': '278905a6-4b52-4d1e-98f9-8c57bb25ba32',
                  'name': 'public_image 2',
                  'status': "active",
                  'owner': TEST.tenant.id,
                  'container_format': 'novaImage',
                  'properties': {'image_type': u'image'},
                  'is_public': True}
    public_image2 = Image(ImageManager(None), image_dict)

    image_dict = {'id': '710a1acf-a3e3-41dd-a32d-5d6b6c86ea10',
                  'name': 'private_image 2',
                  'status': "active",
                  'owner': TEST.tenant.id,
                  'container_format': 'aki',
                  'is_public': False}
    private_image2 = Image(ImageManager(None), image_dict)

    image_dict = {'id': '7cd892fd-5652-40f3-a450-547615680132',
                  'name': 'private_image 3',
                  'status': "active",
                  'owner': TEST.tenant.id,
                  'container_format': 'aki',
                  'is_public': False}
    private_image3 = Image(ImageManager(None), image_dict)

    # A shared image. Not public and not local tenant.
    image_dict = {'id': 'c8756975-7a3b-4e43-b7f7-433576112849',
                  'name': 'shared_image 1',
                  'status': "active",
                  'owner': 'someothertenant',
                  'container_format': 'aki',
                  'is_public': False}
    shared_image1 = Image(ImageManager(None), image_dict)

    # "Official" image. Public and tenant matches an entry
    # in IMAGES_LIST_FILTER_TENANTS.
    image_dict = {'id': 'f448704f-0ce5-4d34-8441-11b6581c6619',
                  'name': 'official_image 1',
                  'status': "active",
                  'owner': 'officialtenant',
                  'container_format': 'aki',
                  'is_public': True}
    official_image1 = Image(ImageManager(None), image_dict)

    TEST.images.add(public_image, private_image,
                    public_image2, private_image2, private_image3,
                    shared_image1, official_image1)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2013 NEC Corporation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django import http
from mox import IsA

from novaclient.v1_1.floating_ip_pools import FloatingIPPool

from vsm_dashboard import api
from vsm_dashboard.test import helpers as test

class NetworkApiNovaFloatingIpTests(test.APITestCase):
    def setUp(self):
        super(NetworkApiNovaFloatingIpTests, self).setUp()
        self.mox.StubOutWithMock(api.base, 'is_service_enabled')
        api.base.is_service_enabled(IsA(http.HttpRequest), 'network') \
            .AndReturn(False)

    def test_floating_ip_pools_list(self):
        pool_names = ['pool1', 'pool2']
        pools = [FloatingIPPool(None, {'name': pool}) for pool in pool_names]
        novaclient = self.stub_novaclient()
        novaclient.floating_ip_pools = self.mox.CreateMockAnything()
        novaclient.floating_ip_pools.list().AndReturn(pools)
        self.mox.ReplayAll()

        ret = api.network.floating_ip_pools_list(self.request)
        self.assertEqual([p.name for p in ret], pool_names)

    def test_floating_ip_list(self):
        fips = self.api_floating_ips.list()
        novaclient = self.stub_novaclient()
        novaclient.floating_ips = self.mox.CreateMockAnything()
        novaclient.floating_ips.list().AndReturn(fips)
        self.mox.ReplayAll()

        ret = api.network.tenant_floating_ip_list(self.request)
        for r, e in zip(ret, fips):
            for attr in ['id', 'ip', 'pool', 'fixed_ip', 'instance_id']:
                self.assertEqual(r.__getattr__(attr), e.__getattr__(attr))
            self.assertEqual(r.port_id, e.instance_id)

    def test_floating_ip_get(self):
        fip = self.api_floating_ips.first()
        novaclient = self.stub_novaclient()
        novaclient.floating_ips = self.mox.CreateMockAnything()
        novaclient.floating_ips.get(fip.id).AndReturn(fip)
        self.mox.ReplayAll()

        ret = api.network.tenant_floating_ip_get(self.request, fip.id)
        for attr in ['id', 'ip', 'pool', 'fixed_ip', 'instance_id']:
            self.assertEqual(ret.__getattr__(attr), fip.__getattr__(attr))
        self.assertEqual(ret.port_id, fip.instance_id)

    def test_floating_ip_allocate(self):
        pool_name = 'fip_pool'
        fip = self.api_floating_ips.first()
        novaclient = self.stub_novaclient()
        novaclient.floating_ips = self.mox.CreateMockAnything()
        novaclient.floating_ips.create(pool=pool_name).AndReturn(fip)
        self.mox.ReplayAll()

        ret = api.network.tenant_floating_ip_allocate(self.request, pool_name)
        for attr in ['id', 'ip', 'pool', 'fixed_ip', 'instance_id']:
            self.assertEqual(ret.__getattr__(attr), fip.__getattr__(attr))
        self.assertEqual(ret.port_id, fip.instance_id)

    def test_floating_ip_release(self):
        fip = self.api_floating_ips.first()
        novaclient = self.stub_novaclient()
        novaclient.floating_ips = self.mox.CreateMockAnything()
        novaclient.floating_ips.delete(fip.id)
        self.mox.ReplayAll()

        api.network.tenant_floating_ip_release(self.request, fip.id)

    def test_floating_ip_associate(self):
        server = api.nova.Server(self.servers.first(), self.request)
        floating_ip = self.floating_ips.first()

        novaclient = self.stub_novaclient()
        novaclient.floating_ips = self.mox.CreateMockAnything()
        novaclient.servers = self.mox.CreateMockAnything()
        novaclient.servers.get(server.id).AndReturn(server)
        novaclient.floating_ips.get(floating_ip.id).AndReturn(floating_ip)
        novaclient.servers.add_floating_ip(server.id, floating_ip.ip) \
                          .AndReturn(server)
        self.mox.ReplayAll()

        api.network.floating_ip_associate(self.request,
                                          floating_ip.id,
                                          server.id)

    def test_floating_ip_disassociate(self):
        server = api.nova.Server(self.servers.first(), self.request)
        floating_ip = self.api_floating_ips.first()

        novaclient = self.stub_novaclient()
        novaclient.servers = self.mox.CreateMockAnything()
        novaclient.floating_ips = self.mox.CreateMockAnything()
        novaclient.servers.get(server.id).AndReturn(server)
        novaclient.floating_ips.get(floating_ip.id).AndReturn(floating_ip)
        novaclient.servers.remove_floating_ip(server.id, floating_ip.ip) \
                          .AndReturn(server)
        self.mox.ReplayAll()

        api.network.floating_ip_disassociate(self.request,
                                             floating_ip.id,
                                             server.id)

    def test_floating_ip_target_list(self):
        servers = self.servers.list()
        novaclient = self.stub_novaclient()
        novaclient.servers = self.mox.CreateMockAnything()
        novaclient.servers.list().AndReturn(servers)
        self.mox.ReplayAll()

        targets = api.network.floating_ip_target_list(self.request)
        for target, server in zip(targets, servers):
            self.assertEqual(target.id, server.id)
            self.assertEqual(target.name, '%s (%s)' % (server.name, server.id))

    def test_floating_ip_target_get_by_instance(self):
        self.mox.ReplayAll()
        instance_id = self.servers.first().id
        ret = api.network.floating_ip_target_get_by_instance(self.request,
                                                             instance_id)
        self.assertEqual(instance_id, ret)

class NetworkApiQuantumFloatingIpTests(test.APITestCase):
    def setUp(self):
        super(NetworkApiQuantumFloatingIpTests, self).setUp()
        self.mox.StubOutWithMock(api.base, 'is_service_enabled')
        api.base.is_service_enabled(IsA(http.HttpRequest), 'network') \
            .AndReturn(True)
        self.qclient = self.stub_quantumclient()

    def test_floating_ip_pools_list(self):
        search_opts = {'router:external': True}
        ext_nets = [n for n in self.api_networks.list()
                    if n['router:external']]
        self.qclient.list_networks(**search_opts) \
            .AndReturn({'networks': ext_nets})
        self.mox.ReplayAll()

        rets = api.network.floating_ip_pools_list(self.request)
        for attr in ['id', 'name']:
            self.assertEqual([p.__getattr__(attr) for p in rets],
                             [p[attr] for p in ext_nets])

    def test_floating_ip_list(self):
        fips = self.api_q_floating_ips.list()
        self.qclient.list_floatingips().AndReturn({'floatingips': fips})
        self.qclient.list_ports().AndReturn({'ports': self.api_ports.list()})
        self.mox.ReplayAll()

        rets = api.network.tenant_floating_ip_list(self.request)
        assoc_port = self.api_ports.list()[1]
        self.assertEqual(len(fips), len(rets))
        for ret, exp in zip(rets, fips):
            for attr in ['id', 'ip', 'pool', 'fixed_ip', 'port_id']:
                self.assertEqual(ret.__getattr__(attr), exp[attr])
            if exp['port_id']:
                dev_id = assoc_port['device_id'] if exp['port_id'] else None
                self.assertEqual(ret.instance_id, dev_id)

    def test_floating_ip_get_associated(self):
        fip = self.api_q_floating_ips.list()[1]
        assoc_port = self.api_ports.list()[1]
        self.qclient.show_floatingip(fip['id']).AndReturn({'floatingip': fip})
        self.qclient.show_port(assoc_port['id']) \
            .AndReturn({'port': assoc_port})
        self.mox.ReplayAll()

        ret = api.network.tenant_floating_ip_get(self.request, fip['id'])
        for attr in ['id', 'ip', 'pool', 'fixed_ip', 'port_id']:
            self.assertEqual(ret.__getattr__(attr), fip[attr])
        self.assertEqual(ret.instance_id, assoc_port['device_id'])

    def test_floating_ip_get_unassociated(self):
        fip = self.api_q_floating_ips.list()[0]
        self.qclient.show_floatingip(fip['id']).AndReturn({'floatingip': fip})
        self.mox.ReplayAll()

        ret = api.network.tenant_floating_ip_get(self.request, fip['id'])
        for attr in ['id', 'ip', 'pool', 'fixed_ip', 'port_id']:
            self.assertEqual(ret.__getattr__(attr), fip[attr])
        self.assertEqual(ret.instance_id, None)

    def test_floating_ip_allocate(self):
        ext_nets = [n for n in self.api_networks.list()
                    if n['router:external']]
        ext_net = ext_nets[0]
        fip = self.api_q_floating_ips.first()
        self.qclient.create_floatingip(
            {'floatingip': {'floating_network_id': ext_net['id']}}) \
            .AndReturn({'floatingip': fip})
        self.mox.ReplayAll()

        ret = api.network.tenant_floating_ip_allocate(self.request,
                                                      ext_net['id'])
        for attr in ['id', 'ip', 'pool', 'fixed_ip', 'port_id']:
            self.assertEqual(ret.__getattr__(attr), fip[attr])
        self.assertEqual(ret.instance_id, None)

    def test_floating_ip_release(self):
        fip = self.api_q_floating_ips.first()
        self.qclient.delete_floatingip(fip['id'])
        self.mox.ReplayAll()

        api.network.tenant_floating_ip_release(self.request, fip['id'])

    def test_floating_ip_associate(self):
        fip = self.api_q_floating_ips.list()[1]
        assoc_port = self.api_ports.list()[1]
        ip_address = assoc_port['fixed_ips'][0]['ip_address']
        target_id = '%s_%s' % (assoc_port['id'], ip_address)
        params = {'port_id': assoc_port['id'],
                  'fixed_ip_address': ip_address}
        self.qclient.update_floatingip(fip['id'],
                                       {'floatingip': params})
        self.mox.ReplayAll()

        api.network.floating_ip_associate(self.request, fip['id'], target_id)

    def test_floating_ip_disassociate(self):
        fip = self.api_q_floating_ips.list()[1]
        assoc_port = self.api_ports.list()[1]
        ip_address = assoc_port['fixed_ips'][0]['ip_address']
        target_id = '%s_%s' % (assoc_port['id'], ip_address)
        self.qclient.update_floatingip(fip['id'],
                                       {'floatingip': {'port_id': None}})
        self.mox.ReplayAll()

        api.network.floating_ip_disassociate(self.request, fip['id'],
                                             target_id)

    def _get_target_id(self, port):
        param = {'id': port['id'],
                 'addr': port['fixed_ips'][0]['ip_address']}
        return '%(id)s_%(addr)s' % param

    def _get_target_name(self, port):
        param = {'svrid': port['device_id'],
                 'addr': port['fixed_ips'][0]['ip_address']}
        return 'server_%(svrid)s: %(addr)s' % param

    def test_floating_ip_target_list(self):
        ports = self.api_ports.list()
        target_ports = [(self._get_target_id(p),
                         self._get_target_name(p)) for p in ports
                        if not p['device_owner'].startswith('network:')]

        self.qclient.list_ports().AndReturn({'ports': ports})
        servers = self.servers.list()
        novaclient = self.stub_novaclient()
        novaclient.servers = self.mox.CreateMockAnything()
        search_opts = {'project_id': self.request.user.tenant_id}
        novaclient.servers.list(True, search_opts).AndReturn(servers)
        self.mox.ReplayAll()

        rets = api.network.floating_ip_target_list(self.request)
        self.assertEqual(len(rets), len(target_ports))
        for ret, exp in zip(rets, target_ports):
            self.assertEqual(ret.id, exp[0])
            self.assertEqual(ret.name, exp[1])

    def test_floating_ip_target_get_by_instance(self):
        ports = self.api_ports.list()
        candidates = [p for p in ports if p['device_id'] == '1']
        search_opts = {'device_id': '1'}
        self.qclient.list_ports(**search_opts).AndReturn({'ports': candidates})
        self.mox.ReplayAll()

        ret = api.network.floating_ip_target_get_by_instance(self.request, '1')
        self.assertEqual(ret, self._get_target_id(candidates[0]))



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import json
import logging

from django.core.urlresolvers import reverse_lazy
from django.http import HttpResponse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tables

from vsm_dashboard.api import vsm as vsmapi
from .form import CreatePool
from .tables import ListPoolTable
from .tables import ListPresentPoolTable
from .utils import list_cinder_service
from .utils import from_keystone_v2
from .utils import from_keystone_v3

LOG = logging.getLogger(__name__)


class ModalEditTableMixin(object):
    def get_template_names(self):
        if self.request.is_ajax():
            if not hasattr(self, "ajax_template_name"):
                # Transform standard template name to ajax name (leading "_")
                bits = list(os.path.split(self.template_name))
                bits[1] = "".join(("_", bits[1]))
                self.ajax_template_name = os.path.join(*bits)
            template = self.ajax_template_name
        else:
            template = self.template_name
        return template

    def get_context_data(self, **kwargs):
        context = super(ModalEditTableMixin, self).get_context_data(**kwargs)
        context['verbose_name'] = getattr(self, "verbose_name", "")
        context['submit_btn'] = getattr(self, "submit_btn", {})
        if self.request.is_ajax():
            context['hide'] = True
        return context

class IndexView(tables.DataTableView):
    table_class = ListPoolTable
    template_name = 'vsm/rbdpools/index.html'

    def get_data(self):
        pools = []
        pool_usages = []
        # TODO pools status
        try:
            pools = vsmapi.pool_list(self.request,)
            pool_usages = vsmapi.pool_usages(self.request)
            logging.debug("resp body in view: %s" % pools)
        except:
            exceptions.handle(self.request,
                              _('Unable to retrieve storage pool list. '))
        pool_usage_dict = {}
        for usage in pool_usages:
            pool_usage_dict.setdefault(str(usage.pool_id), usage)

        for pool in pools:
            pool['id'] = str(pool['id'])
            if pool['id'] in pool_usage_dict:
                pool['attach_status'] = pool_usage_dict[pool['id']].attach_status
            else:
                pool['attach_status'] = "no"

        pools = [x for x in pools if x['tag'] != "SYSTEM" and not x['erasure_code_status']
                     and not str(x['cache_tier_status']).startswith("Cache pool for")]
        return pools

class PresentPoolsView(tables.DataTableView):
    table_class = ListPresentPoolTable
    template_name = 'vsm/rbdpools/rbdsaction.html'
    verbose_name = "Present Pools"

    def get_data(self):
        pools = []
        pool_usages = []
        # TODO pools status
        try:
            pools = vsmapi.pool_list(self.request,)
            pool_usages = vsmapi.pool_usages(self.request)
            logging.debug("resp body in view: %s" % pools)
        except:
            exceptions.handle(self.request,
                              _('Unable to retrieve storage pool list. '))
        pool_usage_dict = {}
        for usage in pool_usages:
            pool_usage_dict.setdefault(str(usage.pool_id), usage)

        for pool in pools:
            pool['id'] = str(pool['id'])
            if pool['id'] in pool_usage_dict:
                pool['attach_status'] = pool_usage_dict[pool['id']].attach_status
            else:
                pool['attach_status'] = "no"

        pools = [x for x in pools if x['tag'] != "SYSTEM"]
        pools = [x for x in pools if x['attach_status'] == "no"]
        pools = [x for x in pools if x['tag'] != "SYSTEM" and not x['erasure_code_status']
             and not str(x['cache_tier_status']).startswith("Cache pool for")]
        return pools

class CreateView(forms.ModalFormView):
    form_class = CreatePool
    template_name = 'vsm/flocking/create.html'
    success_url = reverse_lazy('horizon:vsm:rbdpools:index')

def PoolsAction(request, action):
    data = json.loads(request.body)
    msg = ""
    status = ""
    # glance only need to present one pool
    as_glance_store_pool_num = 0
    for info in data:
        if info["as_glance_store_pool"] == True:
            as_glance_store_pool_num += 1
    if as_glance_store_pool_num > 1:
        msg = "more than one pool as glance backend"
        status = "error"
        resp = dict(message=msg, status=status, data="")
        resp = json.dumps(resp)
        return HttpResponse(resp)

    if as_glance_store_pool_num:
        pool_usages = vsmapi.pool_usages(request)
        for pool_usage in pool_usages:
            print pool_usage.as_glance_store_pool
            if pool_usage.as_glance_store_pool:
                msg = "there is one pool as glance backend now"
                status = "error"
                resp = dict(message=msg, status=status, data="")
                resp = json.dumps(resp)
                return HttpResponse(resp)

    if not len(data):
        status = "error"
        msg = "No pool selected"
    else:
        # TODO add cluster_id in data

        if action == "present":
            print data
            pools = []
            for x in data:
                cinder_volume_host = x['cinder_volume_host']
                if (cinder_volume_host == "" or cinder_volume_host == None) and \
                        (x['as_glance_store_pool'] == "" or x['as_glance_store_pool'] == None):
                    status = "error"
                    msg = "The Cinder Volume Host and As Glance Store Pool are all null"
                pools.append({'pool_id': x['id'],
                              'cinder_volume_host': cinder_volume_host,
                              'appnode_id': x['appnode_id'],
                              'as_glance_store_pool': x['as_glance_store_pool']})
            if msg == "" and status == "":
                print "========Start Present Pools==========="
                result = vsmapi.present_pool(request, pools)
                print result
                host_list = ""
                for host in result['host']:
                    host_list = host + "," + host_list
                if result['status'] == "bad":
                    status = "warning"
                    msg = "Not found crudini commmand in host: %s" % host_list
                elif result['status'] == "unreachable":
                    status = "warning"
                    msg = "Please check ssh with no password between vsm controller and host: %s" % host_list
                else:
                    status = "info"
                    msg = "Begin to Present Pools!"
                print "========End Present Pools==========="

    resp = dict(message=msg, status=status, data="")
    resp = json.dumps(resp)
    return HttpResponse(resp)


def get_openstack_region_select_data(request):
    appnodes = vsmapi.appnode_list(request)
    print "================appnodes get_openstack_region_select_data=========="
    print appnodes
    appnode_list = []
    for appnode in appnodes:
        appnode_dict = {}
        auth_host = appnode.os_auth_url.split(":")[1][2:]
        appnode_dict.update({"display": auth_host + "/" + appnode.os_region_name,
                             "id": appnode.id
                             })
        appnode_list.append(appnode_dict)

    print appnode_list
    data = tuple(appnode_list)
    return HttpResponse(json.dumps(data))


def get_select_data(request):
    print "===================================="
    data = json.loads(request.body)
    print data
    appnode_id  = data["appnode_id"]
    print appnode_id

    service_list = []
    data = {}
    if appnode_id:
        appnodes = vsmapi.appnode_list(request)
        print appnodes
        for appnode in appnodes:
            print appnode.id
            if int(appnode.id) == int(appnode_id):
                print "====================here=============="
                tenant_name = appnode.os_tenant_name
                username = appnode.os_username
                password = appnode.os_password
                auth_url = appnode.os_auth_url
                region_name = appnode.os_region_name
                try:
                    keystone_version = auth_url.strip("/").split("/")[-1]
                    if keystone_version == "v3":
                        tenant_id = tenant_name
                        token, cinder_api_host = from_keystone_v3(tenant_name,
                                                                  username,
                                                                  password,
                                                                  auth_url,
                                                                  region_name)
                    else:
                        token, tenant_id, cinder_api_host = \
                            from_keystone_v2(tenant_name, username, password,
                                             auth_url, region_name)
                    cinder_service_list = list_cinder_service(cinder_api_host, token, tenant_id)

                    cinder_volume_down_list = []
                    for cinder in cinder_service_list:
                        if cinder["state"] == "down":
                            cinder_volume_down_list.append(cinder)

                    if len(cinder_volume_down_list) == 0:
                        for cinder in cinder_service_list:
                            if cinder["binary"] == "cinder-volume":
                                service_list.append(cinder["host"])
                    else:
                        for cinder in cinder_service_list:
                            if cinder["binary"] == "cinder-volume" and \
                                            "@" in cinder["host"]:
                                host = cinder["host"].split("@")[0]
                                if host not in service_list:
                                    service_list.append(host)
                            elif cinder["binary"] == "cinder-volume" and \
                                            "@" not in cinder["host"]:
                                service_list.append(cinder["host"])
                    print service_list
                except:
                    service_list = []

        data = {"host": service_list}
        print data
    return HttpResponse(json.dumps(data))


def get_select_data2(request):
    data = ()
    return HttpResponse(json.dumps(data))



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import tables
from horizon import exceptions
from horizon import messages
from vsm_dashboard.api import vsm as vsmapi

STRING_SEPARATOR = "__"
LOG = logging.getLogger(__name__)

ACTIVE_STATES = ("Present",)
REMOVED_STATES = ("Removed",)
UNINIT_STATES = ("Uninitialized")

class UpdateRow(tables.Row):
    ajax = False

    def get_data(self, request, osd_id):
        osd = vsmapi.osd_get(request, osd_id)
        return osd

class RestartOsdsAction(tables.LinkAction):
    name = "restart_osds"
    verbose_name = _("Restart")
    classes = ('btn-primary',)
    url = "horizon:vsm:devices-management:index"


class RemoveOsdsAction(tables.LinkAction):
    name = "remove_osds"
    verbose_name = _("Remove")
    classes = ('btn-primary',)
    url = "horizon:vsm:devices-management:index"


class RestoreOsdsAction(tables.LinkAction):
    name = "restore_osds"
    verbose_name = _("Restore")
    classes = ('btn-primary',)
    url = "horizon:vsm:devices-management:index"


class AddOsdsAction(tables.BatchAction):
    name = "Add_osds"
    action_present = _("Add")
    action_past = _("Scheduled add of")
    data_type_singular = _("Osd")
    data_type_plural = _("Osds")
    classes = ('btn-primary',)
    redirect_url = "horizon:vsm:devices-management:index"

    def allowed(self, request, osd=None):
        if osd is not None:
            if osd['vsm_status'] not in UNINIT_STATES:
                msg = _('Only osd with VSM status "%s" will be added'%UNINIT_STATES)
                messages.error(request, msg)
                return False
        return True

    def action(self, request, obj_id):
        obj = self.table.get_object_by_id(obj_id)
        name = self.table.get_object_display(obj)
        try:
            vsmapi.add_osd_from_node_in_cluster(request, obj_id)
        except Exception:
            msg = _('Error adding %s.' % name)
            LOG.info(msg)
            redirect = reverse(self.redirect_url)
            exceptions.handle(request, msg, redirect=redirect)

STATUS_DISPLAY_CHOICES = (
    ("removing", "Removing"),
    ("restarting", "Restarting"),
    ("restoring", "Restoring"),
)

class AddOSDAction(tables.LinkAction):
    name = "add_osd"
    verbose_name = _("New")
    url = "/dashboard/vsm/devices-management/add_new_osd/"
    classes = ('btn-primary',)

class OsdsTable(tables.DataTable):
    STATUS_CHOICES = (
        ("Present", True),
        ("Removed", False),
    )
    osd = tables.Column("osd", verbose_name=_("OSD"))
    vsm_status = tables.Column("vsm_status",
                               status=True,
                               status_choices=STATUS_CHOICES,
                               display_choices=STATUS_DISPLAY_CHOICES,
                               verbose_name=_("VSM Status"), hidden=True
                               )
    osd_state = tables.Column("osd_state", verbose_name=_("OSD Status"))
    osd_weight = tables.Column("osd_weight", verbose_name=_("OSD Weight"))
    server = tables.Column("server", verbose_name=_("Server"))
    storage_group = tables.Column("storage_group", \
                                  verbose_name=_("Storage Class"))
    zone = tables.Column("zone", verbose_name=_("Zone"))
    data_dev_path = tables.Column("data_dev_path", \
                                  verbose_name=_("Data Device Path"), hidden=True)
    data_dev_status = tables.Column("data_dev_status", \
                                    verbose_name=_("Data Device Status"))
    data_dev_capacity = tables.Column("data_dev_capacity", \
                                      verbose_name=_("Data Device Capacity (MB)"), hidden=True)
    data_dev_used = tables.Column("data_dev_used", \
                                  verbose_name=_("Data Device Used (MB)"), hidden=True)
    data_dev_available = tables.Column("data_dev_available", \
                                       verbose_name=_("Data Device Available (MB)"), hidden=True)
    journal_device_path = tables.Column("journal_device_path", \
                                        verbose_name=_("Journal Device Path"), hidden=True)
    journal_device_status = tables.Column("journal_device_status", \
                                   verbose_name=_("Journal Device Status"))
    full_status = tables.Column("full_status", \
                               verbose_name=_("Used(%)"), hidden=True)
    full_warn = tables.Column("full_warn", \
                               verbose_name=_("Used Wran"), hidden=True)
    capacity_status = tables.Column("capacity_status", \
                                   verbose_name=_("Capacity Status"))

    class Meta:
        name = "osds"
        verbose_name = _("Device List")
        table_actions = (RestartOsdsAction, RemoveOsdsAction,
                         RestoreOsdsAction,AddOSDAction)
        status_columns = ['vsm_status']
        row_class = UpdateRow

    def get_object_id(self, obj):
        return obj["id"]

    def get_object_by_id(self, obj_id):
        for obj in self.data:
            if self.get_object_id(obj) == int(obj_id):
                return obj
        raise ValueError('No match found for the id "%s".' % obj_id)

    def get_object_display(self, obj):
        return obj["osd"]





# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import os
import sys
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse_lazy

from horizon import exceptions
from horizon import tables
from horizon import forms
from horizon import views
from django import template
from vsm_dashboard.common.horizon.text import TextRenderer
from vsm_dashboard.api import vsm as vsmapi
from .tables import ListServerTable
from .tables import AddServerTable
from .tables import RemoveServerTable
from .tables import AddServerTable2
from .tables import AddMonitorTable
from .tables import RemoveMonitorTable
from .tables import StartServerTable
from .tables import StopServerTable
from django.http import HttpResponse
from .utils import get_server_list
from .utils import get_zone_list,get_zone_not_in_crush_list,get_zone_as_osd_location
from django.views.generic import TemplateView
from django.shortcuts import render,render_to_response




import json
LOG = logging.getLogger(__name__)
from django.core.context_processors import csrf

class ModalEditTableMixin(object):
    def get_template_names(self):
        if self.request.is_ajax():
            if not hasattr(self, "ajax_template_name"):
                # Transform standard template name to ajax name (leading "_")
                bits = list(os.path.split(self.template_name))
                bits[1] = "".join(("_", bits[1]))
                self.ajax_template_name = os.path.join(*bits)
            template = self.ajax_template_name
        else:
            template = self.template_name
        return template

    def get_context_data(self, **kwargs):
        context = super(ModalEditTableMixin, self).get_context_data(**kwargs)
        context['verbose_name'] = getattr(self, "verbose_name", "")
        context['submit_btn'] = getattr(self, "submit_btn", {})
        if self.request.is_ajax():
            context['hide'] = True
        return context

class IndexView(tables.DataTableView):
    table_class = ListServerTable
    template_name = 'vsm/storageservermgmt/index.html'

    def get_data(self):
        servers = get_server_list(self.request, )
        return servers


class AddServersView(tables.DataTableView):
    table_class = AddServerTable
    verbose_name = "Add Servers"
    template_name = 'vsm/storageservermgmt/serversaction.html'

    def get_data(self):
        servers = get_server_list(self.request, lambda x: x['status'] == "available")
        zone_list = get_zone_not_in_crush_list(self.request)

        for server in servers:
            server.update({"zone_list": {"choices": zone_list, "value":server['zone_id']}})
            del server['is_monitor']
            del server['zone']

        return servers


class RemoveServersView(tables.DataTableView):
    table_class = RemoveServerTable
    verbose_name = "Remove Servers"
    template_name = 'vsm/storageservermgmt/serversaction.html'

    def get_data(self):
        return get_server_list(self.request, lambda x: x['status'] in ("Active","unavailable"))



class AddMonitorsView(tables.DataTableView):
    table_class = AddMonitorTable
    template_name = 'vsm/storageservermgmt/serversaction.html'
    verbose_name = "Add Monitors"
    def get_data(self):
        servers = get_server_list(self.request, (
            lambda x: x['status'] in ("Active", "available"),
            lambda x: not (x['status'] == "Active" and "monitor" in x['type']),))
        for server in servers:
            del server['is_monitor']
        return servers


class RemoveMonitorsView(tables.DataTableView):
    table_class = RemoveMonitorTable
    template_name = 'vsm/storageservermgmt/serversaction.html'
    verbose_name = "Remove Monitors"

    def get_data(self):
        servers = get_server_list(self.request, (
            lambda x: x['status'] == "Active",
            lambda x: "monitor" in x['type'],))
        return servers



class StartServersView(tables.DataTableView):
    table_class = StartServerTable
    template_name = 'vsm/storageservermgmt/serversaction.html'
    verbose_name = "Start Servers"
    def get_data(self):
        return get_server_list(self.request, lambda x:x['status'] == "Stopped")


class StopServersView(tables.DataTableView):
    table_class = StopServerTable
    template_name = 'vsm/storageservermgmt/serversaction.html'
    verbose_name = "Stop Servers"
    def get_data(self):
        return get_server_list(self.request, lambda x:x['status'] == "Active")


class ServersActionViewBase(ModalEditTableMixin, tables.DataTableView):
    #success_url = reverse_lazy('horizon:vsm:storageservermgmt:index')

    def get_data(self):
        raise NotImplementedError


class AddServersView2(AddServersView):
    table_class = AddServerTable2

class OsdsText(TextRenderer):
    name = "osds"

    def get_text(self):
        osds = vsmapi.osd_status(self.request)
        osd_list = []
        for _osd in osds:
            osd = {
                "osd_name": _osd.osd_name,
                "server": _osd.service['host'],
                "storage_group": _osd.storage_group['name'],
                "capacity_total": _osd.device['total_capacity_kb'],#TODO dict to obj ?
                "capacity_used": _osd.device['used_capacity_kb'],
                "capacity_avail": _osd.device['avail_capacity_kb'],
                }
            osd_list.append(osd)
        return json.dumps(osd_list) 

class StorageGroupsText(TextRenderer):
    name = "storage_groups"
    def get_text(self):
        storage_groups = vsmapi.storage_group_status(self.request)
        storage_group_list = []
        for _storage_group in storage_groups:
            storage_group = {
                "storage_group_name": _storage_group.name,
                "capacity_total": _storage_group.capacity_total,
                "capacity_used": _storage_group.capacity_used,
                "capacity_avail": _storage_group.capacity_avail,
                }
            storage_group_list.append(storage_group)
        return json.dumps(storage_group_list) 

class StorageGroupThresholdText(TextRenderer):
    name = "storage_group_threshold"
    def get_text(self):
        _cfg = vsmapi.get_setting_dict(self.request)
        threshold = {}
        threshold['full_threshold'] = _cfg["storage_group_full_threshold"]
        threshold['near_full_threshold'] = _cfg["storage_group_near_full_threshold"]
        return json.dumps(threshold)

class TextMixin(object):
    def get_context_data(self, **kwargs):
        context = super(TextMixin, self).get_context_data(**kwargs)
        context['osds_text'] = OsdsText(self.request) 
        context['storage_groups_text'] = StorageGroupsText(self.request)
        context['storage_group_threshold_text'] = StorageGroupThresholdText(self.request)
        return context


def ServersAction(request, action):
    data = json.loads(request.body)

    # TODO add cluster_id in data
    for i in range(0, len(data)):
        data[i]['cluster_id'] = 1
    # TODO add cluster_id in data

    if action == "add":
        vsmapi.add_servers(request, data)
        status = "info"
        msg = "Began to Add Servers"
    elif action == "remove":
        vsmapi.remove_servers(request, data)
        status = "info"
        msg = "Began to Remove Servers"
    elif action == "start":
        vsmapi.start_server(request, data)
        status = "info"
        msg = "Began to Start Servers"
    elif action == "stop":
        vsmapi.stop_server(request, data)
        status = "info"
        msg = "Began to Stop Servers"

    resp = dict(message=msg, status=status, data="")
    resp = json.dumps(resp)
    return HttpResponse(resp)

def ResetStatus(request, server_id):
    resp = dict(message="Began to Reset Status", status="info")
    resp = json.dumps(resp)
    vsmapi.reset_status(request, server_id)
    return HttpResponse(resp)

def update_server_list(request):
    servers = get_server_list(request)
    return HttpResponse(json.dumps(servers))


def AddServerDetailView(request):
    template = "vsm/storageservermgmt/addservers.html"
    context = {}
    servers = get_server_list(request, )
    server_id_list = request.GET.get('id', "").split(",")
    servers_list = [(x["name"],x["name"]+"("+x["primary_public_ip"]+")") for x in servers for y in server_id_list if str(x["id"]) == str(y)]
    context["server_list"] = servers_list
    context["zone_list_server"] = get_zone_not_in_crush_list(request)
    context["zone_list_osd"] = get_zone_as_osd_location(request)
    return render(request,template,context)

def get_server_by_name(request):
    data = json.loads(request.body)
    server_name = data["name"]
    server_list = get_server_list(request,lambda x:x['name'] == server_name)
    server = {}

    paginate_opts = {
        "limit":10000,
        "marker":0,
        "sort_keys":'id',
        "sort_dir":'asc',
        "osd_name":"",
        "server_name":server_name,
        "zone_name":"",
        "state":""
    }

    if(len(server_list)>0):
        osd_list = vsmapi.osd_status_sort_and_filter(request,paginate_opts)
        server = {
            "id":server_list[0]["id"],
            "IP":server_list[0]["primary_public_ip"],
            "name":server_list[0]["name"],
            "zone_id":server_list[0]["zone_id"],
            "is_monitor":server_list[0]["is_monitor"],
            "is_storage":server_list[0]["is_storage"],
            "osd_list":[]
        }
        for osd in osd_list:
            osd_item = {
                "osd_id":osd.id,
                "osd_name":osd.osd_name,
                "zone": osd.zone,
                "weight": osd.weight,
                "journal":osd.device['path'],
                "data": osd.device['journal'],
            }
            server["osd_list"].append(osd_item)

    return HttpResponse(json.dumps(server))

def add_server(request):
    data = json.loads(request.body)
    print "=============Add Server======================"
    print data
    code,ret = vsmapi.add_servers(request, data)
    print ret

    return HttpResponse(json.dumps({"status":"OK"}))


# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2012 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2012 Nebula, Inc.
# Copyright (c) 2012 X.commerce, a business unit of eBay Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import

from django import http
from mox import IsA

from vsm_dashboard import api
from vsm_dashboard.api import vsm
from vsm_dashboard.test import helpers as test
from vsm_dashboard.usage import quotas

class QuotaTests(test.APITestCase):
    @test.create_stubs({api.nova: ('server_list',
                                   'flavor_list',
                                   'tenant_quota_get',),
                        api.network: ('tenant_floating_ip_list',),
                        quotas: ('is_service_enabled',),
                        vsm: ('volume_list', 'tenant_quota_get',)})
    def test_tenant_quota_usages(self):
        quotas.is_service_enabled(IsA(http.HttpRequest),
                                  'volume').AndReturn(True)
        api.nova.flavor_list(IsA(http.HttpRequest)) \
                .AndReturn(self.flavors.list())
        api.nova.tenant_quota_get(IsA(http.HttpRequest), '1') \
                .AndReturn(self.quotas.first())
        api.network.tenant_floating_ip_list(IsA(http.HttpRequest)) \
                .AndReturn(self.floating_ips.list())
        api.nova.server_list(IsA(http.HttpRequest)) \
                .AndReturn(self.servers.list())
        vsm.volume_list(IsA(http.HttpRequest)) \
                .AndReturn(self.volumes.list())
        vsm.tenant_quota_get(IsA(http.HttpRequest), '1') \
            .AndReturn(self.quotas.first())

        self.mox.ReplayAll()

        quota_usages = quotas.tenant_quota_usages(self.request)
        expected_output = {
            'injected_file_content_bytes': {'quota': 1},
            'metadata_items': {'quota': 1},
            'injected_files': {'quota': 1},
            'security_groups': {'quota': 10},
            'security_group_rules': {'quota': 20},
            'fixed_ips': {'quota': 10},
            'gigabytes': {'available': 920, 'used': 80, 'quota': 1000},
            'ram': {'available': 8976, 'used': 1024, 'quota': 10000},
            'floating_ips': {'available': 0, 'used': 2, 'quota': 1},
            'instances': {'available': 8, 'used': 2, 'quota': 10},
            'volumes': {'available': 0, 'used': 3, 'quota': 1},
            'cores': {'available': 8, 'used': 2, 'quota': 10}
        }

        # Compare internal structure of usages to expected.
        self.assertEquals(quota_usages.usages, expected_output)

    @test.create_stubs({api.nova: ('server_list',
                                   'flavor_list',
                                   'tenant_quota_get',),
                        api.network: ('tenant_floating_ip_list',),
                        quotas: ('is_service_enabled',)})
    def test_tenant_quota_usages_without_volume(self):
        quotas.is_service_enabled(IsA(http.HttpRequest),
                                  'volume').AndReturn(False)
        api.nova.flavor_list(IsA(http.HttpRequest)) \
                .AndReturn(self.flavors.list())
        api.nova.tenant_quota_get(IsA(http.HttpRequest), '1') \
                .AndReturn(self.quotas.first())
        api.network.tenant_floating_ip_list(IsA(http.HttpRequest)) \
                .AndReturn(self.floating_ips.list())
        api.nova.server_list(IsA(http.HttpRequest)) \
                .AndReturn(self.servers.list())

        self.mox.ReplayAll()

        quota_usages = quotas.tenant_quota_usages(self.request)
        expected_output = {
            'injected_file_content_bytes': {'quota': 1},
            'metadata_items': {'quota': 1},
            'injected_files': {'quota': 1},
            'security_groups': {'quota': 10},
            'security_group_rules': {'quota': 20},
            'fixed_ips': {'quota': 10},
            'ram': {'available': 8976, 'used': 1024, 'quota': 10000},
            'floating_ips': {'available': 0, 'used': 2, 'quota': 1},
            'instances': {'available': 8, 'used': 2, 'quota': 10},
            'cores': {'available': 8, 'used': 2, 'quota': 10}
        }

        # Compare internal structure of usages to expected.
        self.assertEquals(quota_usages.usages, expected_output)

    @test.create_stubs({api.nova: ('server_list',
                                   'flavor_list',
                                   'tenant_quota_get',),
                        api.network: ('tenant_floating_ip_list',),
                        quotas: ('is_service_enabled',)})
    def test_tenant_quota_usages_no_instances_running(self):
        quotas.is_service_enabled(IsA(http.HttpRequest),
                                  'volume').AndReturn(False)
        api.nova.flavor_list(IsA(http.HttpRequest)) \
                .AndReturn(self.flavors.list())
        api.nova.tenant_quota_get(IsA(http.HttpRequest), '1') \
                .AndReturn(self.quotas.first())
        api.network.tenant_floating_ip_list(IsA(http.HttpRequest)) \
                .AndReturn([])
        api.nova.server_list(IsA(http.HttpRequest)).AndReturn([])

        self.mox.ReplayAll()

        quota_usages = quotas.tenant_quota_usages(self.request)
        expected_output = {
            'injected_file_content_bytes': {'quota': 1},
            'metadata_items': {'quota': 1},
            'injected_files': {'quota': 1},
            'security_groups': {'quota': 10},
            'security_group_rules': {'quota': 20},
            'fixed_ips': {'quota': 10},
            'ram': {'available': 10000, 'used': 0, 'quota': 10000},
            'floating_ips': {'available': 1, 'used': 0, 'quota': 1},
            'instances': {'available': 10, 'used': 0, 'quota': 10},
            'cores': {'available': 10, 'used': 0, 'quota': 10}
        }

        # Compare internal structure of usages to expected.
        self.assertEquals(quota_usages.usages, expected_output)

    @test.create_stubs({api.nova: ('server_list',
                                   'flavor_list',
                                   'tenant_quota_get',),
                        api.network: ('tenant_floating_ip_list',),
                        quotas: ('is_service_enabled',),
                        vsm: ('volume_list', 'tenant_quota_get',)})
    def test_tenant_quota_usages_unlimited_quota(self):
        inf_quota = self.quotas.first()
        inf_quota['ram'] = -1

        quotas.is_service_enabled(IsA(http.HttpRequest),
                                  'volume').AndReturn(True)
        api.nova.flavor_list(IsA(http.HttpRequest)) \
                .AndReturn(self.flavors.list())
        api.nova.tenant_quota_get(IsA(http.HttpRequest), '1') \
                .AndReturn(inf_quota)
        api.network.tenant_floating_ip_list(IsA(http.HttpRequest)) \
                .AndReturn(self.floating_ips.list())
        api.nova.server_list(IsA(http.HttpRequest)) \
                .AndReturn(self.servers.list())
        vsm.volume_list(IsA(http.HttpRequest)) \
                .AndReturn(self.volumes.list())
        vsm.tenant_quota_get(IsA(http.HttpRequest), '1') \
            .AndReturn(inf_quota)

        self.mox.ReplayAll()

        quota_usages = quotas.tenant_quota_usages(self.request)
        expected_output = {
            'injected_file_content_bytes': {'quota': 1},
            'metadata_items': {'quota': 1},
            'injected_files': {'quota': 1},
            'security_groups': {'quota': 10},
            'security_group_rules': {'quota': 20},
            'fixed_ips': {'quota': 10},
            'gigabytes': {'available': 920, 'used': 80, 'quota': 1000},
            'ram': {'available': float("inf"), 'used': 1024,
                    'quota': float("inf")},
            'floating_ips': {'available': 0, 'used': 2, 'quota': 1},
            'instances': {'available': 8, 'used': 2, 'quota': 10},
            'volumes': {'available': 0, 'used': 3, 'quota': 1},
            'cores': {'available': 8, 'used': 2, 'quota': 10}
        }

        # Compare internal structure of usages to expected.
        self.assertEquals(quota_usages.usages, expected_output)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import tables

from vsm_dashboard import api

STRING_SEPARATOR = "__"
LOG = logging.getLogger(__name__)

class DeleteUser(tables.BatchAction):
    name = "delete_user"
    action_present = _("Delete")
    action_past = _("Deleted")
    data_type_singular = _("User")
    data_type_plural = _("Users")
    classes = ('btn-delete',)

    def allowed(self, request, user=None):
        return (user['name'] != "admin")

    def action(self, request, obj_id):
        api.keystone.user_delete(request, obj_id)

class DisableUser(tables.BatchAction):
    name = "disable_user"
    action_present = ("Disable")
    action_past = ("Disabled")
    data_type_singular = _("User")
    data_type_plural = _("Users")
    classes = ('btn-disable')

    def allowed(self, request, user):
        return (user['name'] != "admin" and user['enabled'])

    def action(self, request, obj_id):
        api.keystone.user_update_enabled(request, obj_id, enabled=False)

class EnableUser(tables.BatchAction):
    name = "enable_user"
    action_present = ("Enable")
    action_past = ("Enabled")
    data_type_singular = _("User")
    data_type_plural = _("Users")
    classes = ('btn-enable')

    def allowed(self, request, user):
        return (user['name'] != "admin" and (not user['enabled']))

    def action(self, request, obj_id):
        api.keystone.user_update_enabled(request, obj_id, enabled=True)

class EditUser(tables.LinkAction):
    name = "edit"
    verbose_name = _("Change Password")
    url = "horizon:vsm:usermgmt:update"
    classes = ("ajax-modal", "btn-primary")

    def allowed(self, request, user):
        return api.keystone.keystone_can_edit_user()

class CreateUserAction(tables.LinkAction):
    name = "create user"
    verbose_name = _("Create User")
    url = "horizon:vsm:usermgmt:create"
    classes = ("ajax-modal", "btn-primary")

    def allowed(self, request, datum):
        return (request.user.username == "admin")  

class UserSettingAction(tables.LinkAction):
    name = "set"
    verbose_name = _("Setting")
    url = "horizon:vsm:usermgmt:setting"
    classes = ("ajax-modal", "btn-primary")


class ListUserTable(tables.DataTable):

    user_id = tables.Column("id", verbose_name=_("ID"), classes=("user_list",),hidden=True)
    name = tables.Column("name", verbose_name=_("Name"))
    email = tables.Column("email", verbose_name=_("Email"), hidden=True)
    enabled = tables.Column("enabled", verbose_name="Enabled", hidden=True)

    class Meta:
        name = "user_list"
        verbose_name = _("User List")
        table_actions = (CreateUserAction,)
        row_actions = (EditUser, DeleteUser)
        multi_select = False

    def get_object_id(self, datum):
        if hasattr(datum, "id"):
            return datum.id
        else:
            return datum["id"]

    def get_object_display(self, datum):
        if hasattr(datum, "name"):
            return datum.id
        else:
            return datum["name"]



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
from django.core.urlresolvers import reverse, reverse_lazy



from horizon import forms
from vsm_dashboard.api import vsm as vsmapi
from django.http import HttpResponse
from .form import CephUpgrade

import json
LOG = logging.getLogger(__name__)



class CephUpgradeView(forms.ModalFormView):
    form_class = CephUpgrade
    template_name = 'vsm/cephupgrade/ceph_upgrade.html'
    success_url = reverse_lazy('horizon:vsm:clustermgmt:index')

def ceph_upgrade(request):
    data = json.loads(request.body)
    code,msg = vsmapi.ceph_upgrade(request, data[0])
    status = "info"
    msg = msg.get('message')

    poolusages = vsmapi.pool_usages(request)
    host_list = []
    if poolusages:
        for pool_usage in poolusages:
            volume_host = pool_usage.get("cinder_volume_host")
            if volume_host not in host_list:
                host_list.append(volume_host)
        hosts = ", ".join(host_list)
        msg = msg + "\nPlease upgrade ceph on volume hosts: %s" % hosts

    resp = dict(message=msg, status=status, data="")
    resp = json.dumps(resp)
    return HttpResponse(resp)










# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (c) 2012 OpenStack, LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from os import path

from django.conf import settings

from vsm_dashboard.test import helpers as test

class ErrorPageTests(test.TestCase):
    """ Tests for error pages """
    urls = 'vsm_dashboard.test.error_pages_urls'

    def test_500_error(self):
        TEMPLATE_DIRS = (path.join(settings.ROOT_PATH, 'templates'),)
        with self.settings(TEMPLATE_DIRS=TEMPLATE_DIRS):
            response = self.client.get('/500/')
            self.assertTrue('Server error' in response.content)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import os
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse_lazy

from horizon import exceptions
from horizon import tables
from horizon import forms
from horizon import views

from vsm_dashboard.api import vsm as vsmapi
from .tables import ListPoolStatusTable
from django.http import HttpResponse

import json
LOG = logging.getLogger(__name__)
from vsm_dashboard.utils import get_time_delta

class IndexView(tables.DataTableView):
    table_class = ListPoolStatusTable
    template_name = 'vsm/pool-status/index.html'

    def get_data(self):
        #_pool.= vsmapi.get_pool.list(self.request,)
        _pool_status = []
        try:
            _pool_status = vsmapi.pool_status(self.request)
            if _pool_status:
                logging.debug("resp body in view: %s" % _pool_status)
        except:
            exceptions.handle(self.request,
                              _('Unable to retrieve sever list. '))

        pool_status = []
        for _pool in _pool_status:
            pool = {"id": _pool.poolId,
                        "name": _pool.name,
                        "tag": _pool.tag,
                        "storage_group": _pool.storageGroup,
                        "size": _pool.size,
                        "pg_count": _pool.pgNum,
                        "pgp_count": _pool.pgpNum,
                        "create_by": _pool.createdBy,
                        "kb_used": (_pool.num_bytes / 1024 + 1) if _pool.num_bytes else 0,
                        "objects": _pool.num_objects,
                        "clones": _pool.num_object_clones,
                        "degraded": _pool.num_objects_degraded,
                        "unfound": _pool.num_objects_unfound,
                        "read_ops": _pool.num_read,
                        "read_kb": _pool.num_read_kb,
                        "write_ops": _pool.num_write,
                        "write_kb": _pool.num_write_kb,
                        "client_read_b": _pool.read_bytes_sec,
                        "client_write_b": _pool.write_bytes_sec,
                        "client_ops": _pool.op_per_sec,
                        "status": _pool.status,
                        "updated_at": get_time_delta(_pool.updated_at),
                        }

            pool_status.append(pool)
        pool_status = sorted(pool_status, lambda x,y: cmp(x['id'], y['id']))
        return pool_status

